#!/bin/env perl
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBQIQPzl5jMabHSgyARAgKxAJ9+m2HvdJvlBiGFzR06psVXXLBYkACcDZ2X
#GzffNv3Z/uz1JWIh/Ck6h2s=
#=25n3
#-----END PGP SIGNATURE-----

use strict;
use Getopt::Long;

use File::Basename qw(dirname);
use DE_EPAGES::Core::API::File qw ( GetFileContent);
use DE_EPAGES::Database::API::Config qw( BusinessUnits );
use DE_EPAGES::Core::API::Script qw( RunScript );

sub Main {
    local $| = 1;

    my %Options;
    my @GetOpt = qw(version);
    usage() unless GetOptions(\%Options, @GetOpt);

    chdir dirname $0;
    chdir '..';

    if (not $Options{version}) {
        my $Version = GetFileContent('version.txt');
        chomp $$Version;
        $Options{version} = $$Version;
    }

    my ($Main, $Sub, $Patch, $HF) = split (/\./, $Options{version});

    my $StoreConf = BusinessUnits();
    my $BU = (keys %$StoreConf)[0];
    my $CoreVersion = $StoreConf->{$BU}{Cartridges}{'DE_EPAGES::Core'};

    die "Hotfix $Options{version} can't be applied to DE_EPAGES::Core $CoreVersion.\n".
        "(see $ENV{EPAGES_CONFIG}/Cartridges.d/$BU.conf)\n"
        if $CoreVersion ne "$Main.$Sub.$Patch";

    return if $HF == 1;
    my $PrevHF = "$Main.$Sub.$Patch.". ($HF - 1);
    die "Hotfix $Options{version} can't be applied because previous hotfix $PrevHF not installed yet.\n".
        "(not found: $ENV{EPAGES_CONFIG}/hotfixes.txt)\n"
        unless -f "$ENV{EPAGES_CONFIG}/hotfixes.txt";

    my $InstalledVersions = GetFileContent("$ENV{EPAGES_CONFIG}/hotfixes.txt");

    die "Hotfix $Options{version} can't be applied because already installed.\n".
        "(see $ENV{EPAGES_CONFIG}/hotfixes.txt)\n"
        if $$InstalledVersions =~ /(^|\n)\Q$Options{version}\E(\n|$)/;
    die "Hotfix $Options{version} can't be applied because previous hotfix $PrevHF not installed yet.\n".
        "(see $ENV{EPAGES_CONFIG}/hotfixes.txt)\n"
        if $$InstalledVersions !~ /(^|\n)\Q$PrevHF\E(\n|$)/;
}

RunScript(
    'Sub' => \&Main
);
