#!/bin/sh

#------------------------------------------------------------------------------
# auxiliary functions for epages shell scripts
# -----------------------------------------------------------------------------

# operating system

umask 027
NOHUP=nohup
AWK=awk

export NOHUP AWK
unset LANG LC_ALL LC_CTYPE LC__FASTMSG I_WANT_A_BROKEN_PS

unset EPAGES_KERNEL
UNAME_R=$(uname -r)
cnt=0
for i in ${UNAME_R//[-.]/ } ; do
  EPAGES_KERNEL=$EPAGES_KERNEL.$i
  cnt=$(($cnt + 1))
  [[ $cnt -ge 3 ]] && break
done

if [ -f /etc/redhat-release ] ; then
  EPAGES_KERNEL=${EPAGES_KERNEL}r
elif [ -f /etc/SuSE-release ] ; then
  EPAGES_KERNEL=${EPAGES_KERNEL}s
fi

export EPAGES_KERNEL=${EPAGES_KERNEL#.}

[[ $(uname -p) = x86_64 ]] && EPAGES_ARCH=x86_64 || EPAGES_ARCH=i386
export EPAGES_ARCH

# em(phasize)

export em R G B Y P C
em() {
  local COLOR BGCOLOR OPTIONS
  unset COLOR BGCOLOR OPTIONS
  if [[ $# -le 1 ]] ; then
    OPTIONS='-s(trong) -d(im) -u(nderline) -i(nverse) -h(idden) -x(out)'
    COLOR='-L(black) -R(ed) -G(reen) -B(lue) -Y(ellow) -P(ink) -C(yan) -W(hite)'
    BGCOLOR='-l(black) -r(ed) -g(reen) -b(lue) -y(ellow) -p(ink) -c(yan) -w(hite)'
    echo 'em [OPTIONS] [COLOR] [BGCOLOR] STRING'
    echo "OPTIONS: $OPTIONS"
    echo "COLOR:   $COLOR"
    echo "BGCOLOR: $BGCOLOR"
    return 1
  fi

  COLOR=39 # default
  BGCOLOR=49 # default
  local OPTIND=0
  while getopts sduihxLRGBYPCWlrgbypcw c $@ ; do
    case $c in
      s) OPTIONS="$OPTIONS\\033[1m" ;; # strong
      d) OPTIONS="$OPTIONS\\033[2m" ;; # dim
      u) OPTIONS="$OPTIONS\\033[4m" ;; # underline
      i) OPTIONS="$OPTIONS\\033[7m" ;; # inverse
      h) OPTIONS="$OPTIONS\\033[8m" ;; # hidden
      x) OPTIONS="$OPTIONS\\033[9m" ;; # crossout
      L) COLOR=30 ;; # black
      R) COLOR=31 ;; # red
      G) COLOR=32 ;; # green
      B) COLOR=34 ;; # blue
      Y) COLOR=33 ;; # yellow
      P) COLOR=35 ;; # pink
      C) COLOR=36 ;; # cyan
      W) COLOR=37 ;; # white
      l) BGCOLOR=40 ;; # black
      r) BGCOLOR=41 ;; # red
      g) BGCOLOR=42 ;; # green
      b) BGCOLOR=44 ;; # blue
      y) BGCOLOR=43 ;; # yellow
      p) BGCOLOR=45 ;; # pink
      c) BGCOLOR=46 ;; # cyan
      w) BGCOLOR=47 ;; # white
    esac
  done
  shift $(($OPTIND - 1))

  [[ -z $TERM || $TERM = raw || $TERM = dumb ]] && { echo "$@"; return; }
  echo -en "$OPTIONS\\033[${BGCOLOR};${COLOR}m$@\\033[m"
}

R() { em -R $@ ; }
G() { em -G $@ ; }
B() { em -B $@ ; }
Y() { em -Y $@ ; }
P() { em -P $@ ; }
C() { em -C $@ ; }

# timeout_test

export timeout_test
timeout_test() {
  if [[ $# -le 1 ]] ; then
    echo 'timeout_test FILE_TEST_OPERATOR TEST_OBJECT'
    echo "FILE_TEST_OPERATOR: see 'man test', e.g. one of: -f -d -e -x"
    echo "TEST_OBJECT: file/directory to test"
    return 1
  fi
  local HOW=$1 ; shift
  local TEST=/bin/test
  [ -x /usr/bin/test ] && TEST=/usr/bin/test
  /var/epages/exec.sh -c "$TEST $HOW '$@'"
  return $?
}

[[ $EPAGES_TEST = timeout_test ]] && alias test=timeout_test

# pathmunge

export pathmunge
pathmunge() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'pathmunge [SOURCE_PATH] TARGET_PATH'
    echo "TARGET_PATH: semicolon-separated path to change"
    echo "SOURCE_PATH: semicolon-separated path to prepend"
    echo '$(pathmunge /usr/bin:/bin:/usr/bin) == /usr/bin:/bin'
    echo '$(pathmunge /usr/bin /bin:/usr/bin) == /bin:/usr/bin'
    echo '$(pathmunge /usr/bin /bin)          == /usr/bin:/bin'
    return 1
  fi
  [[ $# -eq 0 ]] && return

  local IN=$2
  local ALL=${IN:+:$IN:}
  local OUT
  local OIFS=$IFS
  IFS=:
  for i in $1 ; do
    [ -z "$i" ] && continue
    test -d "$i" || continue
    [ "${ALL%:$i:*}" = "${ALL#*:$i:}" ] || continue
    ALL="$ALL:$i"
    OUT="$OUT:$i"
  done
  IFS=$OIFS
  [[ -n $IN && -z $OUT ]] && ALL=$IN
  [[ -n $IN && -n $OUT ]] && ALL="${OUT#:}:$IN"
  echo "${ALL#:}"
}

# ipcrm_user

export ipcrm_user
ipcrm_user() {
  if [[ $# -eq 0 || $1 != ${1#-} ]] ; then
    echo 'ipcrm_user USER'
    echo "USER: run ipcrm for UNIX user"
    return 1
  fi
  local user=$1
  [ -z "`ps -u$user -opid= 2>/dev/null`" ] || return
  ipcs -s >/dev/null 2>&1  || return

  ipcs -s | grep $user | while read T ID ELSE ; do ipcrm -s $ID ; done
  ipcs -m | grep $user | while read T ID ELSE ; do ipcrm -m $ID ; done
}

# get_pids

export COLUMNS=1024

export get_pids
get_pids() {
  local PARENTS CHILDREN COMMAND ACCOUNT PIDS PPIDS CPIDS PROCESS
  unset PARENTS CHILDREN COMMAND ACCOUNT PIDS PPIDS CPIDS PROCESS
  local OPTIND=0
  while getopts PCc:u:p:h? c $@ ; do
    case $c in
      P) PARENTS=1 ;;
      C) CHILDREN=1 ;;
      c) COMMAND=$OPTARG ;;
      u) ACCOUNT=$OPTARG ;;
      p) PROCESS=$OPTARG ;;
      [h?]) true ;;
    esac
  done

  if [[ -z "$COMMAND$PROCESS" ]] ; then
    echo 'get_pids {-c COMMAND | -p PID} [-u USER] [-C] [-P]'
    echo "-c COMMAND: returns space separated PID list for command"
    echo "-p PID: returns space separated PID list for PID"
    echo "-u USER: returns only processes of USER"
    echo "-C: add child processes to PID list"
    echo "-P: add parent process to PID list"
    return 1
  fi

  if [[ -n $PROCESS ]] ; then
    PIDS=$(ps -p${PROCESS// /,} -opid= | sort | uniq | tr '\n' ',' | tr -d ' ')
  elif [[ -n $ACCOUNT ]] ; then
    PIDS=$(pgrep -u "$ACCOUNT" -f "$COMMAND" | tr '\n' ',')
  else
    PIDS=$(pgrep -f "$COMMAND" | tr '\n' ',')
  fi
  [[ -z ${PIDS%,} ]] && return

  if [[ -n $PARENTS ]] ; then
    PPIDS=$(ps -p${PIDS%,} -oppid= | sort | uniq | tr '\n' ',' | tr -d ' ')
  fi

  [[ -n $CHILDREN ]] && CPIDS=$(pgrep -P${PIDS%,} | tr '\n' ',')

  PIDS="$PPIDS$PIDS$CPIDS"
  echo ${PIDS//,/ }
}

# show_pids

export show_pids
show_pids() {
  if [[ $1 != ${1#-} ]] ; then
    echo '[PS_FORMAT=...] show_pids PID_LIST'
    echo "PID_LIST: show processes of space separated list in PS_FORMAT"
    return 1
  fi
  [[ $# -eq 0 ]] && return
  local PIDS=$@
  PS_FORMAT=${PS_FORMAT:-user,pid,ppid,nice,pri,pcpu,time,pmem,vsz,etime,args}
  [[ $# -gt 0 ]] && ps -p${PIDS// /,} -o$PS_FORMAT
}

# show_daemon

export show_daemon
show_daemon() {
  if [[ $# -eq 0 || $1 != ${1#-} ]] ; then
    echo 'show_daemon DAEMON'
    echo "DAEMON: show processes of daemon in PS_FORMAT"
    return 1
  fi
  local DAEMON=$1 ; shift
  [[ -z "$DAEMON" ]] && return

  unset PIDS
  case "$EPAGES_KERNEL" in
    # redhat
    *r) LANG=C /etc/init.d/$DAEMON status >/tmp/daemon.~$$~ \
        && PIDS=$(awk '/pid/{gsub(/[^0-9]/,"",$3); print $3}' /tmp/daemon.~$$~) ;;
    # suse
    *s) for pidfile in $(grep -i '/var/.*\.pid' /etc/init.d/$DAEMON  | sed -r 's,.*(/var/(\w|[+,./:=@~-])*).*,\1,' | grep -i '\.pid$') ; do
          [[ -s "$pidfile" ]] || continue
          PIDS=$(cat "$pidfile" | tr -cd '0-9')
          [[ -n $PIDS ]] && break
        done ;;
  esac

  [[ -n ${PIDS//[^0-9]/} ]] && PIDS=$(get_pids -p "$PIDS" $@)
  [[ -n ${PIDS//[^0-9]/} ]] && show_pids "$PIDS" || cat /tmp/daemon.~$$~
  rm -f /tmp/daemon.~$$~
}

# softkill

export softkill
softkill() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'softkill PID_LIST'
    echo "PID_LIST: space separated list of processes to kill; first try TERM, then KILL"
    return 1
  fi
  local PID_LIST=$@
  [[ $# -eq 0 ]] && return

  kill -15 $PID_LIST 2>/dev/null
  sleep 2

  PID_LIST=$(get_pids -p "$PID_LIST" -C)
  [[ -z $PID_LIST ]] && { echo SIGTERM; return; }

  kill -9 $PID_LIST 2>/dev/null
  sleep 2
  echo SIGKILL
}

# started_by_rpm

export started_by_rpm
started_by_rpm() {
  if [[ $# -eq 0 || $1 != ${1#-} ]] ; then
    echo 'started_by_rpm PID'
    echo "returns: 0 if one of ancestors of PID is rpm/yum, else 1"
    return 1
  fi

  USER=${LOGNAME:-$USER}
  ps xwww --forest | tac > /tmp/$USER-started_by_rpm.0
  sed -nr "/^\s+$1\s+/,\$p" /tmp/$USER-started_by_rpm.0 > /tmp/$USER-started_by_rpm.1
  echo PID=$1 >> /tmp/$USER-started_by_rpm.1
  while read -r pid tty stat time cmd ; do
    [[ "$cmd" != "${cmd#*started_by_rpm}" ]] && continue
    [[ "$cmd" != "${cmd#*yum}" || "$cmd" != "${cmd#*rpm}" ]] && return 0
    [[ "$cmd" = "${cmd#*\_}" ]] && break
  done < /tmp/$USER-started_by_rpm.1
  return 1
}

# jre_home

export jre_home
jre_home() {
  if [[ $# -ne 0 ]] ; then
    echo 'jre_home'
    echo "returns: path to JRE directory"
    return 1
  fi
  local JRE EPAGES_JAVA

  case "$EPAGES_KERNEL" in
    *r) JRE=1.6.0-openjdk ;;
    *s) [[ -d /usr/java/latest ]] && { echo /usr/java/latest; return; }
        JRE=1_6_0-ibm ;;
  esac

  local LIB=/usr/lib
  EPAGES_JAVA=$(ls -d $LIB*/jvm/java-$JRE*/jre 2>/dev/null | sort | tail -1)
  [ -n "$EPAGES_JAVA" ] && { echo $EPAGES_JAVA; return; }
  [ -d "$JRE_HOME" ] && { echo $JRE_HOME; return; }

  EPAGES_JAVA=$(ls -d $LIB*/jvm/jre 2>/dev/null | sort | tail -1)
  [ -n "$EPAGES_JAVA" ] && { echo $EPAGES_JAVA; return; }
  EPAGES_JAVA=$(ls -d $LIB*/jvm/java-*/jre 2>/dev/null | sort | tail -1)
  [ -n "$EPAGES_JAVA" ] && { echo $EPAGES_JAVA; return; }
}

# etc_default

export etc_default
etc_default() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'etc_default'
    echo "returns: path to epages default file (e.g. /etc/default/epages6)"
    return 1
  fi
  local PREFIX="$1"
  local PREFIXED=$PREFIX/etc/default/epages6

  [[ -n $PREFIX && -f $PREFIXED ]] && { echo $PREFIXED; return; }

  EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
  [[ -n $EPAGES_PROJECT ]] && { echo /etc/default/$EPAGES_PROJECT; return; }

  PREFIX=$(rpm -q epages-common --qf '[%{INSTPREFIXES}\n]' 2>/dev/null | head -1)
  PREFIXED=$PREFIX/etc/default/epages6
  [[ -n $PREFIX && -f $PREFIXED ]] && { echo $PREFIXED; return; }

  [[ -f /etc/sysconfig/epages6 ]] && . /etc/sysconfig/epages6
  EPAGES_PROJECT=$EPAGES_INSTALL_RUN_SCRIPT
  [[ -n $EPAGES_PROJECT ]] && { echo /etc/default/$EPAGES_PROJECT; return; }

  [[ -f /etc/default/epages6 ]] && { echo /etc/default/epages6; return; }
  [[ $(ls /etc/default/epages6* 2>/dev/null | grep -v '~$' | wc -l) -eq 1 ]] \
  && { ls /etc/default/epages6* | grep -v '~$'; return; }
  echo /etc/default/epages6
}

# run_cron

export EPAGES_CRON_NO_SCRIPT=99
export EPAGES_CRON_IGNORED=98

export run_cron
run_cron() {
  if [[ $# -eq 0 || $1 != ${1#-} ]] ; then
    echo 'run_cron {start|stop|show} [USER]'
    echo "USER: starts/stops/shows epages cron processes (of USER)"
    return 1
  fi
  local RUN=$1
  local user=$2
  local sched="$EPAGES"/bin/epagesScheduler.sh
  local crond=/etc/init.d/cron
  [[ $EPAGES_KERNEL != ${EPAGES_KERNEL%r} ]] && crond=${crond}d

  test -x "$sched" || return $EPAGES_CRON_NO_SCRIPT
  [[ -n $EPAGES_IGNORE_CRON ]] && return $EPAGES_CRON_IGNORED

  local TASK=" ${user:-epages} cron jobs..."
  case "$RUN" in
    start) TASK=$(Y -i Starting $TASK) ;;
    stop)  TASK=$(Y -i Stopping $TASK) ;;
    show)  TASK=$(Y -i Showing $TASK) ;;
    *) echo 'run_cron {start|stop|show} [USER]'
       echo "USER: starts/stops/shows epages cron processes (of USER)"
       return 1 ;;
  esac
  echo $TASK

  $crond status >/dev/null 2>&1 || { $crond status ; return $?; }

  if [[ -n $user ]] ; then
    local SU_EXE=runuser
    $SU_EXE -c true 2>/dev/null || SU_EXE=su
    $SU_EXE $user -c ". \"$(etc_default)\" ; LOGNAME=$user \"$sched\" $RUN" \
    2>/dev/null | grep -v '^S[a-z]*ing '
  else
   "$sched" $RUN | grep -v '^S[a-z]*ing '
  fi

  $crond status
  return $?
}

# mysqld_name

export mysqld_name
mysqld_name() {
  if [[ $# -ne 0 ]] ; then
    echo 'mysqld_name'
    echo "returns: name of MySQL daemon"
    return 1
  fi
  rpm -q --quiet epages-mysqld 2>/dev/null || return
  [[ -w /etc/init.d/mysql ]] && echo mysql || echo mysqld
}

# httpd_name

export httpd_name
httpd_name() {
  if [[ $# -ne 0 ]] ; then
    echo 'httpd_name'
    echo "returns: name of httpd daemon"
    return 1
  fi
  local HTTPD_RPM APACHE2_RPM

  rpm --quiet -q httpd && HTTPD_RPM=1
  rpm --quiet -q apache2 && APACHE2_RPM=1

  [[ -n $HTTPD_RPM && -f /etc/init.d/httpd ]] && { echo httpd; return; }
  [[ -n $APACHE2_RPM && -f /etc/init.d/apache2 ]] && { echo apache2; return; }

  [[ -f /etc/init.d/httpd ]] && { echo httpd; return; }
  [[ -f /etc/init.d/apache2 ]] && { echo apache2; return; }
}

# httpd_confd

export httpd_confd
httpd_confd() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'httpd_confd [epages]'
    echo "returns: path to httpd confdir (with epages configuration)"
    return 1
  fi
  local EPCONF
  [[ $1 = epages ]] && EPCONF=$1

  local NATIVE_HTTPD=$(httpd_name)
  [[ -z $NATIVE_HTTPD ]] && return
  [[ -d /etc/$NATIVE_HTTPD ]] || return

  for i in conf.d vhosts.d extra ; do
    if [[ -f "/etc/$NATIVE_HTTPD/$i/zzz-epages-httpd.conf" ]] ; then
      echo "/etc/$NATIVE_HTTPD/$i" ; return
    elif [[ -d "/etc/$NATIVE_HTTPD/$i" && $EPCONF != epages ]] ; then
      echo "/etc/$NATIVE_HTTPD/$i" ; return
    fi
  done
}

# check nfs client

export check_nfs
check_nfs() {
  if [[ $# -gt 0 ]] ; then
    echo 'check_nfs'
    echo "returns: 0 if NFS client process rpc.statd is running, else 1"
    return 1
  fi
  ps -Crpc.statd || { echo "rpc.statd: no such process"; return 1; }
  return 0
}

# httpd environment

unset EP_HTTPD EPAGES_REWRITE_CONF
if test -d "$EPAGES_APACHE" ; then
  if test -x "$EPAGES_APACHE"/bin/httpd.sh || [[ "$(httpd_confd)" = "" ]] ; then
    HTTPD_ROOT="$EPAGES_APACHE"
    EP_HTTPD="$HTTPD_ROOT/bin/apachectl"
    EPAGES_REWRITE_CONF="$HTTPD_ROOT/conf/extra/zzz-epages-rewrite.conf"
  fi
fi

if [[ -z $EPAGES_REWRITE_CONF ]] ; then
  NATIVE_HTTPD=$(httpd_name)
  CONF_D=$(httpd_confd)
  EPAGES_REWRITE_CONF=${CONF_D:-${HTTPD_ROOT:-/etc/${NATIVE_HTTPD:-httpd}}/conf.d}/zzz-epages-rewrite.conf
fi

export HTTPD_ROOT EP_HTTPD EPAGES_REWRITE_CONF

# perl environment

# perl socket class for https requests
# valid values: Net::SSL or IO::Socket::SSL
PERL_NET_HTTPS_SSL_SOCKET_CLASS=Net::SSL
# verify the hostname in the SSL certificate
# must be set to 0 if PERL_NET_HTTPS_SSL_SOCKET_CLASS=Net::SSL
# PERL_LWP_SSL_VERIFY_HOSTNAME= is same as PERL_LWP_SSL_VERIFY_HOSTNAME=0
PERL_LWP_SSL_VERIFY_HOSTNAME=0

export PERL_NET_HTTPS_SSL_SOCKET_CLASS PERL_LWP_SSL_VERIFY_HOSTNAME

# path

PATH=$(pathmunge "/bin:/usr/bin:/sbin:/usr/sbin" "$PATH")
PATH=$(pathmunge "$PATH:$EPAGES_PERL/bin:$HTTPD_ROOT/bin")
export PATH

export epages_path
epages_path() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'epages_path'
    echo "prepends PATH with EPAGES_PERL/bin"
    return 1
  fi
  pathmunge "$EPAGES_PERL/bin:$PATH"
}

# libraries

LD_LIBRARY_PATH=$(pathmunge "/lib:/usr/lib:/usr/X11R6/lib" "$LD_LIBRARY_PATH:$LD_LIBRARY_PATH_64")
export LD_LIBRARY_PATH

export epages_libs
epages_libs() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'epages_libs'
    echo "prepends LD_LIBRARY_PATH with epages libdirs"
    return 1
  fi
  pathmunge "$EPAGES_PERL/lib32/$EPAGES_KERNEL:$EPAGES_PERL/lib32/ssl:$EPAGES_PERL/bin:$HTTPD_ROOT/lib:$LD_LIBRARY_PATH"
}

# selinux

export set_selinux4epages
set_selinux4epages() {
  if [[ $1 != ${1#-} ]] ; then
    echo 'set_selinux4epages'
    echo "sets SELinux file context for ePages files (if EPAGES_ENABLE_SELINUX is set"
    return 1
  fi
  [[ ${EPAGES_ENABLE_SELINUX:-0} != 0 ]] || return
  test -x $EPAGES/bin/se-file-context.sh || return
  [[ ${EPAGES_ENABLE_PERL_CGI:-0} = 0 ]] && PERL_CGI=--perl-cgi || unset PERL_CGI
  $EPAGES/bin/se-file-context.sh --set $PERL_CGI
}

# check_epages_webgroup

export check_epages_webgroup
check_epages_webgroup() {
  if [[ $# -lt 2 ]] ; then
    echo 'check_epages_web {*start*/*stop*} EP_DEFAULT'
    echo "EP_DEFAULT: checks (e.g. /etc/default/epages6) for EPAGES_WEBGROUP= setting"
    echo "*start*: add missing UNIX user as defined in EPAGES_WEBGROUP="
    echo "returns: GID of epages web group"
    return 1
  fi
  local EP_DEFAULT=$2
  local RESULT

  EPWEBID=${EPAGES_WEBGROUP#*:}
  EPWEB=${EPAGES_WEBGROUP%%:*}
  if [ -z "$EPWEB" ] ; then
    EPWEB=ep_web
    EPWEBID=`$AWK -F: "/^$EPWEB:/{ print \\$3; }" /etc/group`
  elif [ "$EPWEBID" != "${EPWEBID#*:}" ] ; then
    echo "$EP_DEFAULT: EPAGES_WEBGROUP=$EPAGES_WEBGROUP: wrong format of environment variable"
    exit 1
  fi

  case $1 in
  start|*start_httpd|start_ssl|start_service)
    $AWK -F: "/^$EPWEB:/{if (\$3 == \"$EPWEBID\") {exit 123} else exit 124}" /etc/group
    RESULT=$?
    if [ $RESULT = 124 ] ; then
      echo "$EP_DEFAULT: EPAGES_WEBGROUP=$EPAGES_WEBGROUP: void group name:id pair"
      exit 1
    elif [ $RESULT = 0 ] ; then
      groupadd -g $EPWEBID $EPWEB || {
        echo "$0: EPAGES_WEBGROUP=$EPAGES_WEBGROUP: command failed: groupadd -g $EPWEBID $EPWEB"
        exit 1
      }
    fi
    for cfg in group gshadow ; do
      [ -f /etc/$cfg ] || continue
      for user in ep_appl $EPAGES_APPUSER ; do
        egrep -q "^$EPWEB:.*[:,]$user(,|$)" /etc/$cfg || {
          sed -i "s/^\($EPWEB:.*[^:]\)$/\1,$user/;s/^\($EPWEB:.*:\)$/\1$user/" /etc/$cfg || {
            echo "$0: adding user '$user' to /etc/$cfg '$EPWEB' failed"
            exit 1
          }
        }
      done
    done
  ;;
  esac
  echo $EPWEBID
}

#
# END
#
