#!/bin/bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBPXx4vl5jMabHSgyARAn/4AKDF7cnXffaYeTtAn5DfcYuwcco9NQCdF/7A
#NRnixPwcTVDfjetsbnQANiA=
#=b8/2
#-----END PGP SIGNATURE-----

cd $(dirname $0)

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!"
  exit 1
fi

if [ ${0##*/} = uninstall-epages.sh ] ; then
  unset ALLMATCHES
  case "$@" in
    *-\?*|*--help*)  echo "Usage: $0 [rpm erase options]" ; exit 1 ;;
    *--allmatches*) ALLMATCHES=1 ;;
  esac

  EPAGES_INSTALL_PREFIX=`cd ../.. ; pwd`
  PATCH_PKGS=$(rpm -qa | sort -r | awk -F- '/epages-patch_/{printf "-%s ", $2}')
  RPM_LIST=$(for i in "" -setup -sybase -appsrv -reqsrv -mysqld -mysql -sybsrv -sybcli -sybcmn -websrv -webconf -perl -patch $PATCH_PKGS -common -requires -release ; do
    PKG=$(rpm -q --qf '%{NAME}-%{VERSION}-%{RELEASE} [%{INSTPREFIXES}\n]' epages$i)
    [ $? -eq 0 ] && echo "$PKG" | while read PKG INSTALLPREFIX ; do
      [ -n "$ALLMATCHES" -o "$EPAGES_INSTALL_PREFIX" = "$INSTALLPREFIX" -o "x$i" = "x-release" ] && echo $PKG
     done
  done)

  [[ ! -f /etc/sysconfig/epages6 ]] || . /etc/sysconfig/epages6
  EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
  EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}
  . /etc/default/$EPAGES_PROJECT

  FILE_LIST=$(ls -d $EPAGES_INSTALL_PREFIX /var/epages 2>/dev/null)
  for i in default init.d rc?.d sysconfig yum.repos.d ; do
    ls /etc/$i/*epages* >/dev/null 2>&1 && FILE_LIST="$FILE_LIST
/etc/$i/*epages*"
  done

  DB_LIST=$(grep -l 'datasource=dbi:mysql' $EPAGES_CONFIG/Database.d/*.conf 2>/dev/null | xargs grep -l '^StoreType=\S' | sed 's,.*/,,;s,\.conf,,')

  if [ -z "$RPM_LIST$DB_LIST$FILE_LIST" ] ; then
    echo "${0##*/}: ePages not installed."
  else
    [[ -n $FILE_LIST ]] && echo -e "Files:\n$FILE_LIST\n"
    [[ -n $RPM_LIST ]] && echo -e "RPMs:\n$RPM_LIST\n"
    [[ -n $DB_LIST ]] && echo -e "Databases:\n$DB_LIST\n"

    echo -n "Do you really want to delete those ePages files/RPMs/databases? (y/N): "
    read ans
    if [ "$ans" = y -o "$ans" = Y ] ; then
      DROP_DB=$EPAGES_CARTRIDGES/DE_EPAGES/Database/Scripts/dropDB.pl

      if [[ -n $DB_LIST && -x $PERL && -f $DROP_DB ]] ; then
        for i in $DB_LIST ; do
          $PERL $DROP_DB -storename $i
        done
      fi
      if [[ -n $RPM_LIST ]] ; then
        for i in $RPM_LIST ; do
          echo rpm -e "$@" $i
          rpm -e --allmatches "$@" $i
        done
        yum clean all
      fi
      if [[ -n $FILE_LIST ]] ; then
        cd /
        echo "rm -rf $FILE_LIST"
        rm -rf /etc/*/*epages* /srv/epages /var/epages
      fi
    fi
  fi
  exit 0
fi

PATCH_PKGS="`find * -name 'epages-patch_*' -printf '%f ' | sed 's/epages-//g;s/-[0-9][^ ]*//g'`"
DEFAULT_PKGS="release requires common $PATCH_PKGS patch perl mysql appsrv mysqld reqsrv webconf 6"

usage() {
  echo "Usage: $0 [appsrv] [reqsrv] [mysqld] [webconf] [rpm install options]"
  exit 1
}

unset REPLACE PREFIX
OPTIONS="$@"

case "$@" in
  *-\?*|*--help*) usage ;;
  *--replacepkgs*|*--replacefiles*) REPLACE=1 ;;
  *--prefix*) PREFIX=$* ; PREFIX=${PREFIX##*--prefix } ; PREFIX=${PREFIX%% -*} ;;
esac

unset PKG_LIST REQ
while [ $# -gt 0 ] ; do
  case "$1" in
    appsrv|mysqld)  REQ="release requires common $PATCH_PKGS patch perl mysql" ;;
    reqsrv|webconf) REQ="release requires common $PATCH_PKGS patch perl" ;;
    websrv)         REQ="release requires common $PATCH_PKGS patch perl" ;;
    -*) break ;;
    *) usage ;;
  esac
  for j in $REQ $1 ; do
    [[ "$PKG_LIST " == *" "$j" "* ]] || PKG_LIST="$PKG_LIST $j"
  done
  shift
done

[ -n "$PREFIX" ] && EPAGES_INSTALL_PREFIX=$PREFIX
if [ -f /etc/sysconfig/epages6 ] ; then
  [ -n "$PREFIX" ] && sed -i "s,^\([^#]*EPAGES_INSTALL_PREFIX\)=,\1=\"$PREFIX\"," /etc/sysconfig/epages6
  . /etc/sysconfig/epages6
fi
EPAGES_INSTALL_PREFIX=${EPAGES_INSTALL_PREFIX:-/srv/epages}

case `uname -r` in
  2.6.16[.-]*) KERNEL=2.6.16s ;;
  2.6.18[.-]*) KERNEL=2.6.18r ;;
  2.6.27[.-]*) KERNEL=2.6.27s ;;
  2.6.32-*)    KERNEL=2.6.32r ;;
  2.6.32.*)    KERNEL=2.6.32s ;;
esac

unset REQ ALL RELEASE
[[ $(uname -m) = x86_64 ]] && ARCH=x86_64 || ARCH=i386

for i in ${PKG_LIST:-$DEFAULT_PKGS} ; do
  PKG=`ls */epages-$i[.-]*.$ARCH.rpm 2>/dev/null`
  [ -z "$PKG" ] && PKG=`ls */epages-$i[.-]*.rpm kernel/$KERNEL/epages-$i[.-]*.rpm 2>/dev/null`
  [ -z "$PKG" ] && { ls */epages-$i[.-]*.$ARCH.rpm */epages-$i[.-]*.rpm kernel/$KERNEL/epages-$i[.-]*.rpm; exit 1; }
  ALL="$ALL $PKG"
  [ $i = release ] && QF=$EPAGES_INSTALL_PREFIX || QF='[%{INSTPREFIXES}\n]'
  INSTALLPREFIX=$(rpm -q `basename $PKG .rpm` --qf "$QF" | head -1) \
  && [ -z "$REPLACE" -a "$INSTALLPREFIX" = "$EPAGES_INSTALL_PREFIX" ] && continue
  [ $i = release ] && RELEASE=$PKG || REQ="$REQ $PKG"
done

if [ -z "$REQ" ] ; then
  # just show that everything is already installed
  eval rpm -hiv --test --replacefiles $ALL
else
  # install epages packages
  rpm -qa | grep -qi gpg-pubkey-b1d28320- || rpm --import pubring.txt
  export LANG=C
  ARGS="$@"
  if [ -n "$PREFIX" ] ; then
    ARGS_MINUS_PREFIX="$(echo "$*" | perl -pe 's,--prefix(.+?)( -|$),$2,' 2>/dev/null || echo $ARGS)"
    [ -n "$RELEASE" ] && eval rpm -hiv $ARGS_MINUS_PREFIX $RELEASE 2>&1 | tee /tmp/$$.pid
    echo rpm -hiv $ARGS $REQ 2>&1 | tee -a /tmp/$$.pid
  else
    [ -n "$RELEASE" ] && eval rpm -hiv $ARGS $RELEASE 2>&1 | tee -a /tmp/$$.pid
    eval rpm -hiv --prefix $EPAGES_INSTALL_PREFIX $ARGS $REQ 2>&1 | tee -a /tmp/$$.pid
  fi
fi

# -----------------------------------------------------------------------------
# find missing capabilities
# -----------------------------------------------------------------------------

unset REQ

# some capabilities needed for epages packages are missing
for cap in $(awk '/ is needed /{print $1}' /tmp/$$.pid 2>/dev/null) ; do

  case $cap in
    apache2|apache2-worker|audit-libs|bind-utils|coreutils|graphviz) REQ="$REQ $cap" ;;
    gawk|httpd|libstdc++|make|mod_ssl|net-tools|pam|sed|file|gd) REQ="$REQ $cap" ;;
    procps|python-urlgrabber|rpm-python|java-1.6.0-openjdk) REQ="$REQ $cap" ;;
    dejavu-*-fonts) REQ="$REQ $cap" ;;
    jre-openjdk) REQ="$REQ java-1.6.0-openjdk" ;;
    jre-sun) REQ="$REQ java-1_4_2-sun" ;;
    jre-ibm) REQ="$REQ java-1_6_0-ibm" ;;
  esac

  K="$KERNEL-$ARCH"
  case $K in
    2.6.32r-x86_64) I86=.i686 ;;
    2.6.18r-x86_64) I86=.i386 ;;
    *s-*) I86=.i586 ;;
    *) unset I86 ;;
  esac

  case $K in
    *r-*)
    X11=$(basename ${cap%.so.?})
    case $cap in
      libc.so.6) REQ="$REQ glibc" ;;
      libaio.so.1) REQ="$REQ libaio" ;;
      libasound.so.2) REQ="$REQ alsa-lib$I86" ;;
      /lib/libgcc_s.so.1) REQ="$REQ libgcc$I86" ;;
      /lib64/libgcc_s.so.1) REQ="$REQ libgcc" ;;
      /lib/libpam.so.0) REQ="$REQ pam$I86" ;;
      /lib64/libpam.so.0) REQ="$REQ pam" ;;
      /lib/libaudit.so.0) [ $K = ${K#2.6.32} ] || REQ="$REQ audit-libs$I86" ;;
      /lib64/libaudit.so.0) [ $K = ${K#2.6.32} ] || REQ="$REQ audit-libs" ;;
      /usr/lib/libstdc++.so.5) REQ="$REQ  compat-libstdc++-33$I86" ;;
      /usr/lib64/libstdc++.so.5) REQ="$REQ  compat-libstdc++-33" ;;
      /usr/lib/libstdc++.so.6) REQ="$REQ libstdc++$I86" ;;
      /usr/lib64/libstdc++.so.6) REQ="$REQ libstdc++" ;;
      /usr/lib/libstdc++-libc6.2-2.so.3) REQ="$REQ compat-libstdc++-296" ;;
      /sbin/chkconfig) REQ="$REQ chkconfig" ;;
      /usr/bin/crontab) [ $K = ${K#2.6.18} ] && REQ="$REQ cronie" || REQ="$REQ vixie-cron" ;;
      /usr/bin/getconf) REQ="$REQ glibc-common" ;;
      /usr/bin/mysql) REQ="$REQ mysql" ;;
      /usr/bin/mysqld_safe) REQ="$REQ mysql-server" ;;
      /usr/lib/libjpeg.so.62) REQ="$REQ libjpeg$I86" ;;
      /usr/lib/libpng12.so.0) REQ="$REQ libpng$I86" ;;
      /usr/sbin/sendmail) REQ="$REQ sendmail" ;;
      libgssapi_krb5.so.2) REQ="$REQ krb5-libs$I86" ;;
      */libz.so.0) REQ="$REQ zlib$I86" ;;
      /lib/libexpat.so.*) REQ="$REQ expat$I86" ;;
      /usr/lib/libexpat.so.0) REQ="$REQ compat-expat1$I86" ;;
      /usr/lib/libgdbm.so.2) REQ="$REQ gdbm$I86" ;;
      gnupg) [ $K = ${K#2.6.32} ] && REQ="$REQ gnupg" ; [ $K = ${K#2.6.9} ] && REQ="$REQ gnupg2" ;;
      /usr/lib/libxml2.so.2) REQ="$REQ libxml2$I86" ;;
      /usr/lib/libexslt.so.0) REQ="$REQ libxslt$I86" ;;
      /usr/lib/libfreetype.so.6) REQ="$REQ freetype$I86" ;;
      /usr/lib/libfontconfig.so.1) REQ="$REQ fontconfig$I86" ;;
      libX*.so.?|libICE.so.6|libSM.so.6) REQ="$REQ $X11" ;;
      /usr/lib/libX*.so.?) REQ="$REQ $X11$I86" ;;
      /usr/lib/libssl.so.6) [ $K = ${K#2.6.32} ] && REQ="$REQ openssl$I86" || REQ="$REQ openssl098e$I86" ;;
      /usr/lib/libssl.so.10|/lib/libssl.so.6) REQ="$REQ openssl$I86" ;;
      /usr/lib/libgd.so.2) REQ="$REQ gd$I86" ;;
      /lib/libuuid.so.1) [ $K = ${K#2.6.18} ] && REQ="$REQ libuuid$I86" || REQ="$REQ e2fsprogs-libs$I86" ;;
      /usr/lib/libltdl.so.7) REQ="$REQ libtool-ltdl$I86" ;;
    esac ;;
    *s-*)
    [[ $K != ${K#2.6.16} || $cap = libXi.so.6 || $cap = libXtst.so.6 ]] && X11=xorg-x11-libs || X11=xorg-x11-$(basename ${cap%.so.?})
    case $cap in
      libc.so.6) REQ="$REQ glibc" ;;
      libaio.so.1) REQ="$REQ libaio" ;;
      libasound.so.2) [ $K != ${K#2.6.16} ] && REQ="$REQ alsa" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libasound2 libasound2-32bit" || REQ="$REQ libasound2"; } ;;
      /lib/libgcc_s.so.1) [ $K != ${K#2.6.16} ] && REQ="$REQ libgcc" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libgcc43-32bit" || REQ="$REQ libgcc43"; } ;;
      /lib64/libgcc_s.so.1) [ $K != ${K#2.6.16} ] && REQ="$REQ libgcc" || REQ="$REQ libgcc43" ;;
      /lib/libpam.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ pam-32bit" || REQ="$REQ pam" ;;
      /lib64/libpam.so.0) REQ="$REQ pam" ;;
      /lib/libaudit.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ audit-libs-32bit" || REQ="$REQ audit-libs" ;;
      /lib64/libaudit.so.0) REQ="$REQ audit-libs" ;;
      /usr/lib/libstdc++.so.5) [ $K != ${K#2.6.16} ] && REQ="$REQ compat-libstdc++" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libstdc++33-32bit" || REQ="$REQ libstdc++33"; } ;;
      /usr/lib64/libstdc++.so.5) [ $K != ${K#2.6.16} ] && REQ="$REQ compat-libstdc++" || REQ="$REQ libstdc++33" ;;
      /usr/lib/libstdc++.so.6) [ $K != ${K#2.6.16} ] && REQ="$REQ libstdc++" || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libstdc++43-32bit" || REQ="$REQ libstdc++43"; } ;;
      /usr/lib64/libstdc++.so.6) [ $K != ${K#2.6.16} ] && REQ="$REQ libstdc++" || REQ="$REQ libstdc++43" ;;
      /usr/lib/libstdc++-libc6.2-2.so.3) [ $K != ${K%x86_64} ] && REQ="$REQ compat-32bit"  || REQ="$REQ compat" ;;
      /sbin/chkconfig) REQ="$REQ aaa_base" ;;
      /usr/bin/crontab) REQ="$REQ cron" ;;
      /usr/bin/getconf) REQ="$REQ glibc" ;;
      /usr/bin/mysql) REQ="$REQ mysql-client" ;;
      /usr/bin/mysqld_safe) REQ="$REQ mysql" ;;
      /usr/lib/libjpeg.so.62) [ $K != ${K%x86_64} ] && REQ="$REQ libjpeg-32bit" || REQ="$REQ libjpeg" ;;
      /usr/lib/libpng12.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ libpng-32bit" || REQ="$REQ libpng" ;;
      /usr/sbin/sendmail) REQ="$REQ postfix" ;;
      libgssapi_krb5.so.2) [ $K != ${K%x86_64} ] && REQ="$REQ krb5-32bit" || REQ="$REQ krb5" ;;
      */libz.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ zlib-32bit"  || REQ="$REQ zlib" ;;
      /lib/libexpat.so.1) [ $K != ${K%x86_64} ] && REQ="$REQ libexpat1-32bit"  || REQ="$REQ libexpat1" ;;
      /usr/lib/libexpat.so.1) [ $K != ${K%x86_64} ] && REQ="$REQ expat-32bit"  || REQ="$REQ expat" ;;
      gnupg) [ $K = ${K#2.6.16} ] && REQ="$REQ gpg2" || REQ="$REQ gpg" ;;
      /usr/lib/libxml2.so.2) REQ="$REQ libxml2$I86" ;;
      /usr/lib/libexslt.so.0) [ $K != ${K%x86_64} ] && REQ="$REQ libxslt-32bit" || REQ="$REQ libxslt" ;;
      /usr/lib/libfreetype.so.6) [ $K != ${K%x86_64} ] && REQ="$REQ freetype2-32bit" || REQ="$REQ freetype2" ;;
      /usr/lib/libfontconfig.so.1) [ $K != ${K%x86_64} ] && REQ="$REQ fontconfig-32bit" || REQ="$REQ fontconfig" ;;
      libX*.so.?|libICE.so.6|libSM.so.6) REQ="$REQ $X11" ;;
      /usr/lib/libX*.so.?) [ $K != ${K%x86_64} ] && REQ="$REQ $X11-32bit" || REQ="$REQ $X11" ;;
      /usr/lib/libssl.so.0.9.8) [ $K != ${K#2.6.16} ] && {
        [ $K != ${K%x86_64} ] && REQ="$REQ openssl-32bit" || REQ="$REQ openssl"; } || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libopenssl0_9_8-32bit" || REQ="$REQ libopenssl0_9_8"; } ;;
      /lib/libuuid.so.1) [ $K != ${K#2.6.16} ] && {
        [ $K != ${K%x86_64} ] && REQ="$REQ e2fsprogs-32bit" || REQ="$REQ e2fsprogs"; } || {
        [ $K != ${K%x86_64} ] && REQ="$REQ libuuid1-32bit" || REQ="$REQ libuuid1"; } ;;
      /usr/lib/libltdl.so.7) [ $K != ${K%x86_64} ] && REQ="$REQ libltdl7-32bit" || REQ="$REQ libltdl7" ;;
    esac ;;
  esac

done

if [[ -n "$REQ" ]] ; then
  REQ=$(echo $REQ | tr ' ' '\n' | sort | uniq | tr '\n' ' ')
  cp -p /etc/sysconfig/epages6 /etc/sysconfig/epages6.rpmsave 2>/dev/null
  if [[ $KERNEL = ${KERNEL%s} && $REQ != ${REQ#*graphviz} && -f /etc/yum.repos.d/graphviz-rhel.repo ]] ; then
    awk '{if (/^enabled=/ && !X) {sub(/^enabled=.*/,"enabled=1");X=1}; print;}' \
    /etc/yum.repos.d/graphviz-rhel.repo > /etc/yum.repos.d/graphviz-rhel.repo.rpmsave
  fi
  rpm -e epages-release 2>/dev/null
  mv /etc/sysconfig/epages6.rpmsave /etc/sysconfig/epages6 2>/dev/null
  if [[ -f /etc/yum.repos.d/graphviz-rhel.repo.rpmsave ]] ; then
    mv /etc/yum.repos.d/graphviz-rhel.repo.rpmsave /etc/yum.repos.d/graphviz-rhel.repo
  fi
  rm -f /etc/yum.repos.d/*epages-{setup,update,empty}*
  echo
  echo '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  echo "First install missing RPM packages by executing:"
  [ $KERNEL = ${KERNEL%s} ] && INSTALLER=yum || INSTALLER=zypper
  echo "$INSTALLER install $REQ"
  echo "Then re-run:"
  echo "$0 $OPTIONS"
  echo '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  echo
  RET=1
else
  RET=0
fi

rm -f /tmp/$$.pid
exit $RET
