#!/bin/bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBQc8zil5jMabHSgyARAqmkAJ9c3y/u9r+KzQsfubvb3Sf+JgeQXACeLh6u
#ULRdH8F/0g4mROGpvK7zIIw=
#=x5dj
#-----END PGP SIGNATURE-----

[[ "x$1" = "x-pre" ]] && PREREQUISITES=1 || unset PREREQUISITES

umask 022

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!"
  exit 1
fi

EPAGES_INSTALL_LANGUAGES=${EPAGES_LANGUAGES:-$EPAGES_INSTALL_LANGUAGES}
EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
[[ -f /etc/sysconfig/epages6 ]] && . /etc/sysconfig/epages6
EPAGES_INSTALL_LANGUAGES=${EPAGES_INSTALL_LANGUAGES:-$EPAGES_LANGUAGES}
EPAGES_PROJECT=${EPAGES_PROJECT:-$EPAGES_INSTALL_RUN_SCRIPT}
EPAGES_PROJECT=${EPAGES_PROJECT:-epages6}

# -----------------------------------------------------------------------------
# show environment
# -----------------------------------------------------------------------------

show_env () {
  [[ -z $VERBOSE ]] && return
  local LINE_NUMBER=$(printf "%3d" $1)
  shift
  for i in $@ ; do
    if [[ -n $(eval echo '${!'$i'*}') ]] ; then
      echo "$LINE_NUMBER: $i=$(eval echo '$'$i)"
    else
      echo "$LINE_NUMBER: $i=(unset)"
    fi
  done
}

# -----------------------------------------------------------------------------
# set environment
# -----------------------------------------------------------------------------

. /etc/default/$EPAGES_PROJECT
PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin

# -----------------------------------------------------------------------------
# disable epages-update in distributed environment
# -----------------------------------------------------------------------------

if [[ $EPAGES_DISTRIBUTED_INSTALLATION = 1 ]] ; then
  ini-config.sh -f /etc/yum.repos.d/epages-update.repo -s epages-update -k enabled -v 0 -i
fi

# -----------------------------------------------------------------------------
# install or start release
# -----------------------------------------------------------------------------

[[ -f "$EPAGES_CONFIG"/appsrv.ini ]] && NEW_INSTALLATION=1 || NEW_INSTALLATION=

show_env $LINENO NEW_INSTALLATION PREREQUISITES EPAGES_DO_PATCH

if [[ -z $PREREQUISITES && $EPAGES_DO_PATCH != 0 ]] ; then
  EPAGES_IGNORE_CRON=1 /etc/init.d/$EPAGES_PROJECT start || exit $?
fi

# -----------------------------------------------------------------------------
# find current version
# -----------------------------------------------------------------------------

if ls "$EPAGES_CONFIG"/Cartridges.d/*.conf >/dev/null 2>&1 ; then
  CONF="$EPAGES_CONFIG"/Cartridges.d/\*.conf
  CONFFILE="$EPAGES_CONFIG"/Cartridges.d/*.conf
else
  exit 0
fi

CURRENT_VERSION=$(awk -F= '/^DE_EPAGES::Core=/{print $2;exit 0}' $CONFFILE)
unset CURRENT_REL
while IFS=. read -a i ; do CURRENT_REL=(${i[@]}) ; done <<<$CURRENT_VERSION

if [[ -z ${CURRENT_REL[2]} ]] ; then
  echo "Install failed: $CONF: DE_EPAGES::Core=$CURRENT_VERSION: format must be X.Y.Z"
  exit 1
fi

if [[ ${CURRENT_REL[0]} -ne 6 ]] ; then
  echo "Install failed: $CONF: DE_EPAGES::Core=$CURRENT_VERSION: must be 6.X.Y"
  exit 1
fi

if [[ ${CURRENT_REL[1]} -lt 9 ]] ; then
  echo "Install failed: $CONF: DE_EPAGES::Core=$CURRENT_VERSION: must be at least 6.9.X"
  exit 1
fi

show_env $LINENO CONF CONFFILE CURRENT_VERSION

# -----------------------------------------------------------------------------
# disable epages-setup.repo
# -----------------------------------------------------------------------------

if [ -f /etc/yum.repos.d/epages-setup.repo ] ; then
  ini-config.sh -f /etc/yum.repos.d/epages-setup.repo -s epages-setup -k enabled -v 0 -i
fi

# -----------------------------------------------------------------------------
# find latest version
# -----------------------------------------------------------------------------

EPAGES_PATCHES=${EPAGES_PATCHES:-"$EPAGES_SHARED"/Patches}

cd "$EPAGES_PATCHES" >/dev/null 2>&1 && ls -d patch_* >/dev/null 2>&1 
[[ $? -eq 0 ]] && HAS_PATCH=1 || unset HAS_PATCH

if [[ -n $HAS_PATCH ]] ; then
  LATEST_REL=(0 0 0)
  for PATCH_TAG in patch_* ; do
    PATCH_TAG=${PATCH_TAG#patch_} 
    [[ -n ${PATCH_TAG//[0-9_]/} ]] && continue

    while IFS=_ read -a i ; do j=(${i[@]}) ; done <<<${PATCH_TAG#patch_}
    [[ -z ${j[2]} ]] && continue

    if [[ ( ${j[0]} -gt ${LATEST_REL[0]} ) || ( ${j[0]} -eq ${LATEST_REL[0]} && ${j[1]} -gt ${LATEST_REL[1]} ) || \
          ( ${j[0]} -eq ${LATEST_REL[0]} && ${j[1]} -eq ${LATEST_REL[1]} && ${j[2]} -gt ${LATEST_REL[2]} ) ]] ; then
      LATEST_REL=( ${j[@]} )
    fi
  done
fi

[ -f "$EPAGES"/Service/epagesService ] && INSTALL=install || INSTALL=preinstall

[[ -n ${!EPAGES_BACKUP_DB*} ]] && SET_BACKUP_DB=BACKUP_DB=$EPAGES_BACKUP_DB || SET_BACKUP_DB=
[[ -n ${!EPAGES_BACKUP_DIR*} ]] && SET_BACKUP_DIR=BACKUP_DIR=$EPAGES_BACKUP_DIR || SET_BACKUP_DIR=

show_env $LINENO EPAGES_PATCHES HAS_PATCH PATCH_TAG INSTALL EPAGES_BACKUP_DB EPAGES_BACKUP_DIR SET_BACKUP_DB SET_BACKUP_DIR

# -----------------------------------------------------------------------------
# check prerequisits
# -----------------------------------------------------------------------------

show_env $LINENO PREREQUISITES HAS_PATCH INSTALL EPAGES_DO_PATCH

if [[ -n $PREREQUISITES ]] ; then
  [[ -n $HAS_PATCH ]] || exit 0
  [[ $INSTALL = install ]] || exit 0
  [[ $EPAGES_DO_PATCH = 0 ]] && exit 0

  i=${CURRENT_REL[1]}
  j=${CURRENT_REL[2]}
  unset EPAGES_LANG_VERSIONS

  # check minor releases

  while [[ $i -lt ${LATEST_REL[1]} ]] ; do
    j=0
    i=$(($i + 1))
    PATCH_TAG=patch_${LATEST_REL[0]}_${i}_$j
    ls "$EPAGES_PATCHES"/$PATCH_TAG >/dev/null || continue
    EPAGES_LANG_VERSIONS="$EPAGES_LANG_VERSIONS ${LATEST_REL[0]}.$i.$j"
  done

  # check patch releases

  while [[ $j -lt ${LATEST_REL[2]} ]] ; do
    j=$(($j + 1))
    PATCH_TAG=patch_${LATEST_REL[0]}_${LATEST_REL[1]}_$j
    ls "$EPAGES_PATCHES"/$PATCH_TAG >/dev/null || continue
    EPAGES_LANG_VERSIONS="$EPAGES_LANG_VERSIONS ${LATEST_REL[0]}.${LATEST_REL[1]}.$j"
  done

  export EPAGES_VALID_VERSION=$CURRENT_VERSION
  export EPAGES_LANG_VERSIONS

  PATCH_TAG=patch_${LATEST_REL[0]}_${LATEST_REL[1]}_${LATEST_REL[2]}

  [ -f "$EPAGES_PATCHES"/$PATCH_TAG/patch.sh ] && PATCHIT=./patch.sh || PATCHIT='$PERL patch.pl'
  if cd "$EPAGES_PATCHES"/$PATCH_TAG ; then
    eval $PATCHIT get_var $SET_BACKUP_DB $SET_BACKUP_DIR || exit 1
    rm -f host-*.mk
  fi

  exit 0
fi

show_env $LINENO EPAGES_LANG_VERSIONS EPAGES_VALID_VERSION PATCH_TAG PATCHIT

# -----------------------------------------------------------------------------
# run patch?
# -----------------------------------------------------------------------------

show_env $LINENO HAS_PATCH EPAGES_DO_PATCH

if [[ -n $HAS_PATCH && -z ${!EPAGES_DO_PATCH*} ]] ; then
  IFCONFIG=$(ip addr show | awk '/ inet6? /{sub(/\/.*/,"",$2);printf "%s ", $2}' 2>/dev/null)

  $PERL -e '
use Sys::Hostname;
use Sys::HostIP;
use IO::Socket;
use DE_EPAGES::Core::API::EpagesConfig qw(EpagesConfig);
use DE_EPAGES::WebInterface::API::Config qw(WebInterfaceConfig);
use DE_EPAGES::Core::API::PerlTools qw(GetIPAddresses);

my $IPs = Sys::HostIP->ips;
push @$IPs, inet_ntoa( scalar( gethostbyname( hostname())));
push @$IPs, "::1";
push @$IPs, @{GetIPAddresses()->{6}};
push @$IPs, @ARGV;

my $SystemAddr = EpagesConfig->get("DE_EPAGES::Object", "SystemDomainName", "localhost");
$SystemAddr = inet_ntoa(scalar( gethostbyname($SystemAddr) ))
    unless $SystemAddr =~ /^[0-9a-fA-F:.]+$/;
foreach my $TestIP ($SystemAddr, @{WebInterfaceConfig->sectionNames}) {
    next unless $TestIP =~ /^[0-9a-fA-F:.]+$/;
    exit 1 unless $TestIP ~~ @$IPs;
}
' $IFCONFIG >/dev/null 2>&1 || EPAGES_DO_PATCH=0
fi

# -----------------------------------------------------------------------------
# install all patches
# -----------------------------------------------------------------------------

export IGNORE_RUNNING_JOBS=1

show_env $LINENO IGNORE_RUNNING_JOBS HAS_PATCH IFCONFIG

if [[ -n $HAS_PATCH ]] ; then
  i=${CURRENT_REL[1]}
  j=${CURRENT_REL[2]}

  # install minor releases

  while [[ $i -lt ${LATEST_REL[1]} ]] ; do
    j=0
    i=$(($i + 1))
    PATCH_TAG=patch_${LATEST_REL[0]}_${i}_$j
    ls "$EPAGES_PATCHES"/$PATCH_TAG >/dev/null || continue
    [ -f "$EPAGES_PATCHES"/$PATCH_TAG/patch.sh ] && PATCHIT=./patch.sh || PATCHIT='$PERL patch.pl'
    if [[ $EPAGES_DO_PATCH = 0 ]] ; then
      echo "Now run: cd \$EPAGES_PATCHES/$PATCH_TAG && $PATCHIT $INSTALL $SET_BACKUP_DB $SET_BACKUP_DIR"
    else
      cd "$EPAGES_PATCHES"/$PATCH_TAG && { eval $PATCHIT $INSTALL $SET_BACKUP_DB $SET_BACKUP_DIR || exit 1; }
    fi
  done

  # install patch releases

  while [[ $j -lt ${LATEST_REL[2]} ]] ; do
    j=$(($j + 1))
    PATCH_TAG=patch_${LATEST_REL[0]}_${LATEST_REL[1]}_$j
    ls "$EPAGES_PATCHES"/$PATCH_TAG >/dev/null || continue
    [ -f "$EPAGES_PATCHES"/$PATCH_TAG/patch.sh ] && PATCHIT=./patch.sh || PATCHIT='$PERL patch.pl'
    if [[ $EPAGES_DO_PATCH = 0 ]] ; then
      echo "Now run: cd \$EPAGES_PATCHES/$PATCH_TAG && $PATCHIT $INSTALL $SET_BACKUP_DB $SET_BACKUP_DIR"
    else
      cd "$EPAGES_PATCHES"/$PATCH_TAG && { eval $PATCHIT $INSTALL $SET_BACKUP_DB $SET_BACKUP_DIR || exit 1; }
    fi
  done
fi

show_env $LINENO EPAGES_DO_PATCH NEW_INSTALLATION EPAGES_INSTALL_LANGUAGES

[[ $EPAGES_DO_PATCH = 0 || -z $NEW_INSTALLATION || -z $EPAGES_INSTALL_LANGUAGES ]] && exit 0

show_env $LINENO PERL EPAGES_CARTRIDGES EPAGES_INSTALL_LANGUAGES

$PERL "$EPAGES_CARTRIDGES"/DE_EPAGES/Installer/Scripts/addLanguages.pl -lang "$EPAGES_INSTALL_LANGUAGES"

exit $?
