#!/bin/bash
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBQfBm9l5jMabHSgyARAnjQAJ4+r5TYx7OoARODf6uaBue9AieSqwCeJ8Fi
#kYWoN2oJINUcfz4YuGwBkWI=
#=/NuG
#-----END PGP SIGNATURE-----

BN=${0##*/}

umask 022
export LANG=C

if [[ ! -w /etc/passwd ]] ; then
  echo "Error: Only 'root' can execute this program!"
  exit 1
fi

# -----------------------------------------------------------------------------
# find epages environment
# -----------------------------------------------------------------------------

VARIABLES_SOURCE=environment
INSTALL_VERSION=${EPAGES_INSTALL_VERSION:-$EPAGES_INSTALL_ACCEPT_VERSION}

if [[ -z $EPAGES_PROJECT && -f /etc/sysconfig/epages6 ]] ; then
   . /etc/sysconfig/epages6
  VARIABLES_SOURCE=sysconfig
  EPAGES_PROJECT=$EPAGES_INSTALL_RUN_SCRIPT
fi

# ignore EPAGES_INSTALL_VERSION in /etc/sysconfig/epages6
EPAGES_INSTALL_VERSION=$INSTALL_VERSION

if [[ -z $EPAGES_PROJECT ]] ; then
  VARIABLES_SOURCE=default
  EPAGES_PROJECT=epages6
fi

unset COMMON_RPM_INSTALLED
if rpm --quiet -q epages-common ; then
  EPAGES_PARENT="$(rpm -q epages-common --qf '[%{VERSION} %{INSTPREFIXES}\n]' | head -1)"
  COMMON_RPM_INSTALLED="${EPAGES_PARENT%% *}"
  EPAGES_PARENT="${EPAGES_PARENT#* }"
fi

EPAGES_INSTALLED=1
if [[ -f /etc/default/$EPAGES_PROJECT ]] ; then
  . /etc/default/$EPAGES_PROJECT
else
  EPAGES="${EPAGES_PARENT:-/srv/epages}"/eproot
  EPAGES_SHARED="$EPAGES"/Shared
  [[ -n $COMMON_RPM_INSTALLED ]] && EPAGES_CONFIG="$EPAGES_SHARED"/Config || unset EPAGES_INSTALLED
fi

# -----------------------------------------------------------------------------
# evaluate command line, set:
# EPAGES_INSTALL_VERSION : as defined in environment, format e.g. 6.14.1
# EPAGES_RPMS_URL : either default or as defined with -repo
# -----------------------------------------------------------------------------

EPAGES_RPMS_URL=ftp://epages-software.de/pub/products/RPMS
KERNEL_REPO=/etc/yum.repos.d/epages-kernel.repo
[[ -f /etc/yum.repos.d/epages-arch.repo ]] && KERNEL_REPO=/etc/yum.repos.d/epages-arch.repo

usage() {
  echo "Usage:"
  echo "  ${0##*/} [-repo [REPO]] [-nodeps]"
  echo "Options:"
  echo "  -repo: use other ePages repository, not $EPAGES_RPMS_URL"
  echo "         without REPO use repository as before (defd in $KERNEL_REPO)"
  echo "  -nodeps: do not check ePages RPMs, just update /etc/yum.repos.d"
  exit 1
}

ARGUMENTS=$@
unset OWN_REPO NO_DEPS
while [[ $# -gt 0 ]] ; do
  case $1 in
    --repo|-repo) OWN_REPO=-repo ; [[ -n $2 && ${2#-} = $2 ]] && { shift; OWN_REPO="$1"; } ;;
    --nodeps|-nodeps) NO_DEPS=1 ;;
    *) usage; break ;;
  esac
  shift
done

if [[ -n $OWN_REPO ]] ; then
  case $OWN_REPO in
    -repo) ls $KERNEL_REPO >/dev/null || usage
      OWN_REPO=$(grep '^[[:space:]]*baseurl' $KERNEL_REPO | head -1 | tr -d '[\042\047[:space:]]' | sed 's,[^=]*=,,;s,/kernel/.*,,')
      [[ -z $OWN_REPO ]] && { echo "$BN: baseurl not found in: $KERNEL_REPO" ; usage; }
    ;;
    http://*|https://*|ftp://*|file://*) true
    ;;
    *) OWN_REPO=$(cd $OWN_REPO && pwd) || { echo "$NB: error: -repo: invalid argument value"; usage; }
      OWN_REPO=file://$OWN_REPO
    ;;
  esac
  EPAGES_RPMS_URL=$OWN_REPO
fi

# -----------------------------------------------------------------------------
# set version variables:
# SOURCE_VER  : version of DE_EPAGES::Core in Cartridges.d/*.conf, format e.g. 6.14.1
# TARGET_VER  : either EPAGES_INSTALL_VERSION or version from epages-version.noarch.rpm
# INSTALL_VER : either SOURCE_VER or TARGET_VER, format e.g. 6.14.1
# INTERNAL_TARGET_VER : as TARGET_VER, format e.g. 6014001
# INTERNAL_INSTALL_VER : as INSTALL_VER, format e.g. 6014001
# UPDATE_VER  : major version of INSTALL_VER, format e.g. 6.14
# PATCH_TAG   : CVS tag of INSTALL_VER, format e.g. patch_6_14_1
# COMMON_RPM_INSTALLED : version of installed epages-common, format e.g. 6.14.1
# -----------------------------------------------------------------------------

EPAGES_RPMS_URL=$(echo "$EPAGES_RPMS_URL" | sed -r 's,/+$,,')

rpm -qa | grep -qi gpg-pubkey-b1d28320- || rpm --import $EPAGES_RPMS_URL/rpmify/RPM-GPG-KEY-epages || exit 1

TARGET_VER=$(rpm -qp "$EPAGES_RPMS_URL/epages-version.noarch.rpm" --qf '[%{VERSION}\n]') || usage
[[ $EPAGES_INSTALL_VERSION = latest ]] && EPAGES_INSTALL_VERSION=$TARGET_VER

if [[ -n $EPAGES_INSTALL_VERSION ]] ; then
  TARGET_VER=$EPAGES_INSTALL_VERSION
  RELEASE_RPM="$EPAGES_RPMS_URL/$TARGET_VER/epages-release.noarch.rpm"
else
  RELEASE_RPM="$EPAGES_RPMS_URL/epages-release.noarch.rpm"
fi

echo -n "Installing: "
rpm -qp $RELEASE_RPM || exit 1

CARTRIDGES_CONF="\$EPAGES_CONFIG/Cartridges.d/\*.conf"

unset SOURCE_VER
if [[ -n $EPAGES_INSTALLED ]] ; then

  echo -n "ePages found"
  [[ -n $COMMON_RPM_INSTALLED ]] && echo " (epages-common: $COMMON_RPM_INSTALLED)" \
  || echo " (/etc/default/$EPAGES_PROJECT)"

  # following lines necessary for old epages versions (starting from 6.10.0)
  CONFFILES="$EPAGES_CONFIG"/Cartridges.d/*.conf
  if grep -q '^\[Cartridges\]' "$EPAGES_CONFIG"/Database.d/*.conf >/dev/null 2>&1 ; then
    CONFFILES="$EPAGES_CONFIG"/Database.d/*.conf
  elif ls "$EPAGES_CONFIG"/Cartridges.d/*.conf >/dev/null 2>&1 ; then
    CONFFILES="$EPAGES_CONFIG"/Cartridges.d/*.conf
  elif [[ -f "$EPAGES_CONFIG"/Cartridges.conf ]] ; then
    CONFFILES="$EPAGES_CONFIG"/Cartridges.conf
  fi

  if ! ls $CONFFILES >/dev/null 2>&1 ; then
    echo -e "Error: Install failed, file(s) do not exist:\n  $CARTRIDGES_CONF"
    exit 1
  fi

  SOURCE_VER=$(awk -F= '/^DE_EPAGES::Core=/{print $2;exit 0}' $CONFFILES)
  if [[ -z $SOURCE_VER ]] ; then
    echo "Error: Install failed: Can't find DE_EPAGES::Core= in '$CONFFILES'."
    exit 1
  fi
  echo "ePages found (DE_EPAGES::Core=$SOURCE_VER)"
fi

INSTALL_VER=${SOURCE_VER:-$TARGET_VER}

unset INTERNAL_TARGET_VER REL
while IFS=. read -a i ; do REL=(${i[@]}) ; done <<<$TARGET_VER
if [[ -n ${REL[2]} ]] ; then
  INTERNAL_TARGET_VER=${REL[0]}$(printf '%03d%03d' ${REL[1]} ${REL[2]})
fi

unset REL
while IFS=. read -a i ; do REL=(${i[@]}) ; done <<<$INSTALL_VER
INTERNAL_INSTALL_VER=0
if [[ -n ${REL[2]} ]] ; then
  INTERNAL_INSTALL_VER=${REL[0]}$(printf '%03d%03d' ${REL[1]} ${REL[2]})
fi

UPDATE_VER=${REL[0]}.${REL[1]}
INTERNAL_TARGET_VER=${INTERNAL_TARGET_VER:-$INTERNAL_INSTALL_VER}

PATCH_TAG=patch_${REL[0]}_${REL[1]}_${REL[2]}
unset REL

[[ -n $EPAGES_INSTALLED ]] && MSG="$CARTRIDGES_CONF: DE_EPAGES::Core=" || MSG="repository $EPAGES_RPMS_URL: "
if [[ $INTERNAL_INSTALL_VER -lt 6009000 ]] ; then
  echo "Error: Install failed: $MSG$INSTALL_VER: void version"
  exit 1
fi

# -----------------------------------------------------------------------------
# check kernel version
# -----------------------------------------------------------------------------

[[ $(uname -p) = x86_64 ]] && ARCH=x86_64 || ARCH=i386

case $(uname -r) in
  2.6.16[.-]*) KERNEL=2.6.16s ;;
  2.6.18[.-]*) KERNEL=2.6.18r ;;
  2.6.27[.-]*) KERNEL=2.6.27s ;;
  2.6.32-*)    KERNEL=2.6.32r ;;
  2.6.32.*)    KERNEL=2.6.32s ;;
  3.0.*) KERNEL=2.6.32s ;;
  *) echo "Error: Install failed: `uname -r`: Linux kernel version not supported."
     exit 1 ;;
esac

# -----------------------------------------------------------------------------
# check database type
# -----------------------------------------------------------------------------

if [[ -z $NO_DEPS ]] ; then

  cat $EPAGES_CONFIG/Database.conf $EPAGES_CONFIG/Database.d/*.conf 2>/dev/null \
  | grep -q '^[[:space:]]*datasource[[:space:]]*=[[:space:]]*dbi:mysql' \
  && MYSQL_CLIENT=1 || unset MYSQL_CLIENT

  unset DB_CLIENT_RPMS SERVER_RPMS

  if [[ -n $MYSQL_CLIENT ]] ; then
    MY_IPS=$(LANG=C ifconfig -a | awk '/addr:/{sub(/.*addr: */,"",$0);sub(/[/ ].*/,"",$0);print}')
    for file in $(ls $EPAGES_CONFIG/Database.conf $EPAGES_CONFIG/Database.d/*.conf 2>/dev/null) ; do
      datasource=$(grep '^[[:space:]]*datasource[[:space:]]*=[[:space:]]*dbi:mysql' $file \
                   | sed 's/.*datasource[[:space:]]*=[[:space:]]*//;s/["''"]//g;' | head -1)
      [[ -z $datasource ]] && continue
      host=${datasource##*hostname=}
      host=${host##*host=}
      if [[ $host != $datasource ]] ; then
        host=${host%%;*}
        if [[ ${host//:/} = $host && -n ${host//[0-9.]/} ]] ; then
          host=$(dig +short $host +search +noall +answer | head -1)
          [[ -z $host ]] && continue
        fi
        for i in $MY_IPS ; do
          [[ $i = $host ]] && { SERVER_RPMS=mysqld ; break; }
        done
      else
        MY_CNF=$(grep '^[[:space:]]*MySQLConfigFile[[:space:]]*=' $file \
                 | sed 's/.*MySQLConfigFile[[:space:]]*=[[:space:]]*//;s/["''"]//g;' | head -1)
        [[ -f ${MY_CNF:-/etc/my.cnf} ]] && { SERVER_RPMS=mysqld ; break; }
      fi
    done
  fi

  if [[ -n $MYSQL_CLIENT && ( -x $EPSERVICE || $SERVER_RPMS = mysqld ) ]] ; then
    DB_CLIENT_RPMS="$DB_CLIENT_RPMS mysql"
  fi

fi

# -----------------------------------------------------------------------------
# find installed servers
# -----------------------------------------------------------------------------

unset MOD_RPAF
if [[ $KERNEL = ${KERNEL%s} ]] ; then
  INSTALLER=yum ; YES=-y ; unset ZES
  [[ $INTERNAL_TARGET_VER -ge 6014000 ]] && MOD_RPAF=mod_rpaf
else
  INSTALLER=zypper ; ZES=-y ; unset YES
  [[ $INTERNAL_TARGET_VER -ge 6014000 ]] && MOD_RPAF=apache2-mod_rpaf
fi

unset OTHER_RPMS
if [[ -z $NO_DEPS ]] ; then

  [ -f /etc/SuSE-release ] && HTTPD_RPM=apache2 || HTTPD_RPM=httpd
  if find /etc/{apache2,httpd} -name zzz-epages-httpd.conf 2>/dev/null | grep -q . && rpm --quiet -q $HTTPD_RPM ; then
    SERVER_RPMS="$SERVER_RPMS webconf" ; OTHER_RPMS=$MOD_RPAF
  fi

  [[ -x $EP_RR ]] && SERVER_RPMS="$SERVER_RPMS reqsrv"
  [[ -d $EPAGESJ ]] && SERVER_RPMS="$SERVER_RPMS j"
  [[ -x $EP_HTTPD ]] && SERVER_RPMS="$SERVER_RPMS websrv"
  [[ -x $EPSERVICE ]] && SERVER_RPMS="$SERVER_RPMS appsrv"
fi

# -----------------------------------------------------------------------------
# legacy: create symlink /srv/epages if necessary
# -----------------------------------------------------------------------------

EPAGES_PARENT=/srv/epages

if [[ -z $NO_DEPS && -n $EPAGES_INSTALLED ]] ; then
  [[ -d $EPAGES ]] && EPAGES_PARENT=$(cd $EPAGES/.. ; pwd) || EPAGES_PARENT=$(dirname $EPAGES)

  if [[ -d $EPAGES_PARENT/ && $(find $EPAGES/bin -maxdepth 0 -printf "%i") != $(find /srv/epages/eproot/bin -maxdepth 0 -printf "%i" 2>/dev/null) ]] ; then
    DATE=.~$(date +%s)~
    rm -f /srv/epages/{docs,etc,eproot} /srv/epages 2>/dev/null
    for i in $(find /srv/epages/{*,.[^.]*} -maxdepth 0 -printf "%f " 2>/dev/null) ; do
      ][ -f $(readlink -f $EPAGES_PARENT/$i) ]] || rm -f $EPAGES_PARENT/$i 2>/dev/null
      [[ -e $EPAGES_PARENT/$i ]] && mv /srv/epages/$i $EPAGES_PARENT/$i$DATE
      [[ -e $EPAGES_PARENT/$i ]] || mv /srv/epages/$i $EPAGES_PARENT/
    done
    rmdir /srv/epages 2>/dev/null
    rm -f /srv/epages 2>/dev/null
    mv /srv/epages /srv/epages$DATE 2>/dev/null
    [[ -d /srv ]] || mkdir -m 755 /srv
    ( cd /srv ; ln -s $EPAGES_PARENT epages )
  fi
fi

# -----------------------------------------------------------------------------
# find missing rpms to install (empty version)
# -----------------------------------------------------------------------------

[[ -n $EPAGES_INSTALL_VERSION ]] && echo "Using EPAGES_INSTALL_VERSION: $EPAGES_INSTALL_VERSION"
echo "Using repository: $EPAGES_RPMS_URL"
echo "Using install prefix: $EPAGES_PARENT"
echo "Using project name: $EPAGES_PROJECT"

[[ $VARIABLES_SOURCE = sysconfig ]] && echo "Using sysconfig variables: /etc/sysconfig/epages6"
[[ -f /etc/default/$EPAGES_PROJECT ]] && echo "Using default variables: /etc/default/$EPAGES_PROJECT"

PATCH_RPMS=$PATCH_TAG
[[ ${PATCH_TAG%_0} = $PATCH_TAG ]] && PATCH_RPMS="${PATCH_TAG%_*}_0 $PATCH_RPMS"

CHECK_RPMS="release requires common $PATCH_RPMS patch perl $DB_CLIENT_RPMS $SERVER_RPMS"
[[ -n $NO_DEPS ]] && CHECK_RPMS=release

unset REQ 
for i in $CHECK_RPMS ; do
  rpm --quiet -q epages-$i || {
    case $i in
      release) true ;;
      requires) REQ="$REQ $EPAGES_RPMS_URL/$INSTALL_VER/base/$KERNEL/epages-$i-$KERNEL-0.i386.rpm" ;;
      webconf) REQ="$REQ $EPAGES_RPMS_URL/rpmify/kernel/epages-$i-$KERNEL-0.i386.rpm" ;;
      ${PATCH_TAG%_*}_0) REQ="$REQ $EPAGES_RPMS_URL/rpmify/epages-patch/epages-$i-${INSTALL_VER%.*}.0-0.i386.rpm" ;;
      $PATCH_TAG) REQ="$REQ $EPAGES_RPMS_URL/rpmify/epages-patch/epages-$i-$INSTALL_VER-0.i386.rpm" ;;
      *) REQ="$REQ $EPAGES_RPMS_URL/rpmify/epages-$i/epages-$i-$INSTALL_VER-0.i386.rpm" ;;
    esac
  }
done

[[ -z $EPAGES_INSTALLED ]] && unset REQ

# -----------------------------------------------------------------------------
# repository functions
# -----------------------------------------------------------------------------

# third party capabilities
cap3rd=(graphviz nodejs mp3splt mongo-10gen-server mod_rpaf)
# are provided by third party repositories
repo3rd=(graphviz-rhel nodejs-rhel atrpms 10gen CentALT)
[[ $KERNEL = ${KERNEL%s} ]] || repo3rd=("" nodejs-sles packman-multimedia 10gen "")

edit_repos() {
  for i in `seq 0 $((${#cap3rd[@]} -1))` ; do
    [[ -z ${repo3rd[$i]} ]] && continue
    [[ -f /etc/yum.repos.d/${repo3rd[$i]}.repo ]] || continue
    cp -p /etc/yum.repos.d/${repo3rd[$i]}.repo /etc/yum.repos.d/${repo3rd[$i]}.repo.rpmsave
    case ${repo3rd[$i]}-$KERNEL in
      10gen*) [[ $ARCH != x86_64 ]] && sed -i 's,\$basearch,i686,g' /etc/yum.repos.d/${repo3rd[$i]}.repo ;;
      nodejs-sles-2.6.27s) sed -i 's,_SP1,,g' /etc/yum.repos.d/${repo3rd[$i]}.repo ;;
      nodejs-rhel-2.6.18r) sed -i 's,\$releasever,5,g' /etc/yum.repos.d/${repo3rd[$i]}.repo ;;
      nodejs-rhel-2.6.32r) sed -i 's,\$releasever,6,g' /etc/yum.repos.d/${repo3rd[$i]}.repo ;;
      CentALT-2.6.18r) sed -i 's,\$releasever,5,g' /etc/yum.repos.d/${repo3rd[$i]}.repo ;;
      CentALT-2.6.32r) sed -i 's,\$releasever,6,g' /etc/yum.repos.d/${repo3rd[$i]}.repo ;;
    esac
  done
}

restore_repos() {
  rpm --nodeps -e epages-release 2>/dev/null
  rpm --quiet -q epages-release 2>/dev/null || {
    # delete files only if epages-release does not exist anymore
    # reset %config(noreplace)
    [ -f /etc/sysconfig/epages6.rpmsave ] && mv /etc/sysconfig/epages6.rpmsave /etc/sysconfig/epages6
    [ -f /var/epages/mongod.rpmsave ] && mv /var/epages/mongod.rpmsave /var/epages/mongod
    for i in ${repo3rd[@]} ; do
      [ -f /etc/yum.repos.d/$i.rpmsave ] && mv /etc/yum.repos.d/$i.rpmsave /etc/yum.repos.d/$i
    done
    rm -f /etc/yum.repos.d/*epages-{setup,update,empty}*
  }
}

add_zypper_repos() {
  if cd /etc/zypp/repos.d 2>/dev/null ; then
    for i in ${repo3rd[@]} $@ ; do
      [[ $i = ${i%.repo} ]] && i=$i.repo
      [[ ! -f "/etc/yum.repos.d/$i" ]] && continue
      rm -f $i
      ln -fs "/etc/yum.repos.d/$i"
    done
  fi
}

# -----------------------------------------------------------------------------
# install epages-release rpm
# -----------------------------------------------------------------------------

RELEASE_RPM_INSTALLED=$(rpm -q epages-release --qf '[%{VERSION}\n]' 2>/dev/null | egrep '^[0-9.]+$' | head -1)

echo "Old version (epages-release): $RELEASE_RPM_INSTALLED"
echo "New version (epages-release): $TARGET_VER"
rm -f /tmp/${BN%.sh}.log

if [[ $RELEASE_RPM_INSTALLED = $TARGET_VER ]] ; then
  rpm -V --nouser --nogroup --nomtime --nomode epages-release \
  | awk '{if ($2 !~ /^.$/) print}' | grep . && unset RELEASE_RPM_INSTALLED
fi

if [[ $RELEASE_RPM_INSTALLED != $TARGET_VER ]] ; then
  restore_repos
  rpm -hUv $@ $RELEASE_RPM 2>&1 | tee -a /tmp/${BN%.sh}.log
  RELEASE_RPM_INSTALLED=$(rpm -q epages-release --qf '[%{VERSION}\n]' 2>/dev/null | egrep '^[0-9.]+$' | head -1)
fi

if [[ -f /etc/pki/rpm-gpg/RPM-GPG-KEY-epages ]] ; then
  rpm -qa | grep -qi gpg-pubkey-b1d28320- || rpm --import /etc/pki/rpm-gpg/RPM-GPG-KEY-epages
fi

# -----------------------------------------------------------------------------
# install other missing rpms (empty version)
# -----------------------------------------------------------------------------

if [[ -z $NO_DEPS && -n $REQ ]] ; then
  eval rpm -hiv $@ --prefix $EPAGES_PARENT $REQ 2>&1 | tee -a /tmp/${BN%.sh}.log
  [[ ${PIPESTATUS[0]} = 0 ]] || exit 1
fi

# -----------------------------------------------------------------------------
# find missing capabilities of empty rpms
# -----------------------------------------------------------------------------

get_req() {
  local K=$1
  local cap=$2
  local SV=${K%%?-*} ; SV=${SV##*.}
  SV=$(eval echo '$'S${SV##*.})

  [[ $K != ${K#2.6.16} || $K = ${K%x86_64} ]] && { echo ${SV:-$S}; return; }
  [[ -z $cap ]] && { echo ${SV:-$S} ${SV:-$S}-32bit; return; }
  [[ $cap = ${cap#*lib64} ]] && echo ${SV:-$S}-32bit || echo ${SV:-$S}
}

unset REQ

# some capabilities needed for epages packages are missing
for cap in $(awk '/ is needed /{print $1}' /tmp/${BN%.sh}.log 2>/dev/null) ; do

  case $cap in
    apache2|apache2-worker|audit-libs|bind-utils|coreutils) REQ="$REQ $cap" ;;
    gawk|httpd|libstdc++|make|mod_ssl|net-tools|pam|sed|file|gd) REQ="$REQ $cap" ;;
    procps|python-urlgrabber|rpm-python|java-1.6.0-openjdk) REQ="$REQ $cap" ;;
    graphviz|nodejs|mp3splt|mongo-10gen-server|mod_rpaf|apache2-mod_rpaf) REQ="$REQ $cap" ;;
    dejavu-*-fonts) REQ="$REQ $cap" ;;
    jre-openjdk) REQ="$REQ java-1.6.0-openjdk" ;;
    jre-sun) REQ="$REQ java-1_4_2-sun" ;;
    jre-ibm) REQ="$REQ java-1_6_0-ibm" ;;
  esac

  case `uname -r` in
    3.0.*) K="2.6.32s-$ARCH" ;;
    *) K="$KERNEL-$ARCH" ;;
  esac
  case $K in
    2.6.32r-x86_64) I86=.i686 ;;
    2.6.18r-x86_64) I86=.i386 ;;
    *s-*) I86=.i586 ;;
    *) unset I86 ;;
  esac

  case $K in
    *r-*)
    X11=$(basename ${cap%.so.?})
    case $cap in
      libc.so.6) REQ="$REQ glibc" ;;
      libaio.so.1) REQ="$REQ libaio" ;;
      libasound.so.2) REQ="$REQ alsa-lib$I86" ;;
      /lib/libgcc_s.so.1) REQ="$REQ libgcc$I86" ;;
      /lib64/libgcc_s.so.1) REQ="$REQ libgcc" ;;
      /lib/libpam.so.0) REQ="$REQ pam$I86" ;;
      /lib64/libpam.so.0) REQ="$REQ pam" ;;
      /lib/libaudit.so.0) [ $K = ${K#2.6.32} ] || REQ="$REQ audit-libs$I86" ;;
      /lib64/libaudit.so.0) [ $K = ${K#2.6.32} ] || REQ="$REQ audit-libs" ;;
      /usr/lib/libstdc++.so.5) REQ="$REQ  compat-libstdc++-33$I86" ;;
      /usr/lib64/libstdc++.so.5) REQ="$REQ  compat-libstdc++-33" ;;
      /usr/lib/libstdc++.so.6) REQ="$REQ libstdc++$I86" ;;
      /usr/lib64/libstdc++.so.6) REQ="$REQ libstdc++" ;;
      /usr/lib/libstdc++-libc6.2-2.so.3) REQ="$REQ compat-libstdc++-296" ;;
      /sbin/chkconfig) REQ="$REQ chkconfig" ;;
      /usr/bin/crontab) [ $K = ${K#2.6.18} ] && REQ="$REQ cronie" || REQ="$REQ vixie-cron" ;;
      /usr/bin/getconf) REQ="$REQ glibc-common" ;;
      /usr/bin/mysql) REQ="$REQ mysql" ;;
      /usr/bin/mysqld_safe) REQ="$REQ mysql-server" ;;
      /usr/lib/libjpeg.so.62) REQ="$REQ libjpeg$I86" ;;
      /usr/lib/libpng12.so.0) REQ="$REQ libpng$I86" ;;
      /usr/sbin/sendmail) REQ="$REQ sendmail" ;;
      libgssapi_krb5.so.2) REQ="$REQ krb5-libs$I86" ;;
      */libz.so.0) REQ="$REQ zlib$I86" ;;
      /lib/libexpat.so.*) REQ="$REQ expat$I86" ;;
      /usr/lib/libexpat.so.0) REQ="$REQ compat-expat1$I86" ;;
      /usr/lib/libgdbm.so.2) REQ="$REQ gdbm$I86" ;;
      gnupg) [ $K = ${K#2.6.32} ] && REQ="$REQ gnupg" ; [ $K = ${K#2.6.9} ] && REQ="$REQ gnupg2" ;;
      /usr/lib/libxml2.so.2) REQ="$REQ libxml2$I86" ;;
      /usr/lib/libexslt.so.0) REQ="$REQ libxslt$I86" ;;
      /usr/lib/libfreetype.so.6) REQ="$REQ freetype$I86" ;;
      /usr/lib/libfontconfig.so.1) REQ="$REQ fontconfig$I86" ;;
      libX*.so.?|libICE.so.6|libSM.so.6) REQ="$REQ $X11" ;;
      /usr/lib/libX*.so.?) REQ="$REQ $X11$I86" ;;
      /usr/lib/libssl.so.6) [ $K = ${K#2.6.32} ] && REQ="$REQ openssl$I86" || REQ="$REQ openssl098e$I86" ;;
      /usr/lib/libssl.so.10|/lib/libssl.so.6) REQ="$REQ openssl$I86" ;;
      /usr/lib/libgd.so.2) REQ="$REQ gd$I86" ;;
      /lib/libuuid.so.1) [ $K = ${K#2.6.18} ] && REQ="$REQ libuuid$I86" || REQ="$REQ e2fsprogs-libs$I86" ;;
      /usr/lib/libltdl.so.7) REQ="$REQ libtool-ltdl$I86" ;;
    esac ;;
    *s-*)
    unset S S16 S27 S32 S13
    [[ $K != ${K#2.6.16} || $cap = libXi.so.6 || $cap = libXtst.so.6 ]] && X11=xorg-x11-libs || X11=xorg-x11-$(basename ${cap%.so.?})
    case $cap in
      libc.so.6) REQ="$REQ glibc" ;;
      libaio.so.1) REQ="$REQ libaio" ;;
      libasound.so.2) S16=alsa ; S=libasound2 ; REQ="$REQ $(get_req $K)" ;;
      /lib*/libgcc_s.so.1) S16=libgcc ; S13=libgcc46 ; S=libgcc43 ; REQ="$REQ $(get_req $K $cap)" ;;
      /lib*/libpam.so.0) S=pam ; REQ="$REQ $(get_req $K $cap)" ;;
      /lib*/libaudit.so.0) S=audit-libs ; REQ="$REQ $(get_req $K $cap)" ;;
      /usr/lib*/libstdc++.so.5) S16=compat-libstdc++ ; S=libstdc++33 ; REQ="$REQ $(get_req $K $cap)" ;;
      /usr/lib*/libstdc++.so.6) S16=libstdc++ ; S13=libstdc++46 ; S=libstdc++43 ; REQ="$REQ $(get_req $K $cap)" ;;
      /usr/lib/libstdc++-libc6.2-2.so.3) S=compat ; REQ="$REQ $(get_req $K $cap)" ;;
      /sbin/chkconfig) REQ="$REQ aaa_base" ;;
      /usr/bin/crontab) REQ="$REQ cron" ;;
      /usr/bin/getconf) REQ="$REQ glibc" ;;
      /usr/bin/mysql) REQ="$REQ mysql-client" ;;
      /usr/bin/mysqld_safe) REQ="$REQ mysql" ;;
      /usr/lib/libjpeg.so.62) S=libjpeg ; REQ="$REQ $(get_req $K $cap)" ;;
      /usr/lib/libpng12.so.0) S13=libpng12 ; S=libpng ; REQ="$REQ $(get_req $K $cap)" ;;
      /usr/sbin/sendmail) REQ="$REQ postfix" ;;
      libgssapi_krb5.so.2) S=krb5 ; REQ="$REQ $(get_req $K $cap)" ;;
      */libz.so.0) S=zlib ; REQ="$REQ $(get_req $K ${cap##*/})" ;;
      /lib/libexpat.so.1) S=libexpat1 ; REQ="$REQ $(get_req $K $cap)" ;;
      /usr/lib/libexpat.so.1) S=expat ; REQ="$REQ $(get_req $K $cap)" ;;
      gnupg) [ $K = ${K#2.6.16} ] && REQ="$REQ gpg2" || REQ="$REQ gpg" ;;
      /usr/lib/libxml2.so.2) REQ="$REQ libxml2$I86" ;;
      /usr/lib/libexslt.so.0) S=libxslt ; REQ="$REQ $(get_req $K $cap)" ;;
      /usr/lib/libfreetype.so.6) S=freetype2 ; REQ="$REQ $(get_req $K $cap)" ;;
      /usr/lib/libfontconfig.so.1) S=fontconfig ; REQ="$REQ $(get_req $K $cap)" ;;
      libX*.so.?|libICE.so.6|libSM.so.6) REQ="$REQ $X11" ;;
      /usr/lib/libX*.so.?) S=$X11 ; REQ="$REQ $(get_req $K $cap)" ;;
      /usr/lib/libssl.so.0.9.8) S16=openssl ; S=libopenssl0_9_8 ; REQ="$REQ $(get_req $K $cap)" ;;
      /lib/libuuid.so.1) S16=e2fsprogs ; S=libuuid1 ; REQ="$REQ $(get_req $K $cap)" ;;
      /usr/lib/libltdl.so.7) S=libltdl7 ; REQ="$REQ $(get_req $K $cap)" ;;
    esac ;;
  esac

done

# -----------------------------------------------------------------------------
# abort if prerequisites missing
# -----------------------------------------------------------------------------

edit_repos

REQ=$(echo $REQ | tr ' ' '\n' | sort | uniq | tr '\n' ' ' | sed 's,^ *,,;s, *$,,')

if [[ -n "$REQ" ]] ; then
  restore_repos
  add_zypper_repos
  echo
  echo '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  echo "First install missing RPM packages by executing:"
  echo "$INSTALLER install $REQ"
  echo "Then re-run:"
  echo "$0 $ARGUMENTS"
  echo '!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
  echo
  exit 1
fi

# -----------------------------------------------------------------------------
# legacy: create /var/tmp/ini-config.sh and /var/tmp/epages-fs.sh
# -----------------------------------------------------------------------------

mkdir -m 755 -p /var/tmp
cp --remove-destination -s /var/epages/ini-config.sh /var/tmp/ini-config.sh 2>/dev/null
cp --remove-destination -s /var/epages/epages-fs.sh /var/tmp/epages-fs.sh 2>/dev/null

# -----------------------------------------------------------------------------
# create missing repositories
# -----------------------------------------------------------------------------

cd /etc/yum.repos.d || exit 1

rm -f epages-dev.repo epages.repo mirrors-epages epages-kernel.repo mirrors-epages-kernel-*

# create epages-arch.repo
repo=epages-arch.repo

if [[ ! -f $repo ]] ; then
  echo '### Name: ePages RPM Repository' > $repo
  echo '### URL: http://epages.com/' >> $repo
  echo -e "[epages-arch]\nname = epages.com - arch" >> $repo
  echo -e "baseurl = $EPAGES_RPMS_URL/kernel/$ARCH" >> $repo
  echo "mirrorlist = file:///etc/yum.repos.d/mirrors-epages-arch-$ARCH" >> $repo
  echo -e "enabled = 1\ntype = rpm-md\ngpgcheck = 1" >> $repo
  echo -e "gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-epages" >> $repo
  echo -e "\n[epages-arch-i386]\nname = epages.com - arch" >> $repo
  echo -e "baseurl = $EPAGES_RPMS_URL/kernel/i386" >> $repo
  echo "mirrorlist = file:///etc/yum.repos.d/mirrors-epages-arch-i386" >> $repo
  echo -e "enabled = 1\ntype = rpm-md\ngpgcheck = 1" >> $repo
  echo -e "gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-epages" >> $repo
  echo -e "\n[epages-arch-kernel]\nname = epages.com - arch-kernel" >> $repo
  echo -e "baseurl = $EPAGES_RPMS_URL/kernel/$ARCH/$KERNEL" >> $repo
  echo "mirrorlist = file:///etc/yum.repos.d/mirrors-epages-arch-$ARCH-$KERNEL" >> $repo
  echo -e "enabled = 1\ntype = rpm-md\ngpgcheck = 1" >> $repo
  echo -e "gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-epages" >> $repo
fi

for i in i386 x86_64 ; do
  if [[ ! -f mirrors-epages-arch-$i ]] ; then
    echo "http://www.epages.com/RPMS/kernel/$i" >  mirrors-epages-arch-$i
    echo "http://ep-server/RPMS/kernel/$i" >>  mirrors-epages-arch-$i
  fi
  for j in 2.6.16s 2.6.18r 2.6.27s 2.6.32r 2.6.32s ; do
    if [[ ! -f mirrors-epages-arch-$i-$j ]] ; then
      echo "http://www.epages.com/RPMS/kernel/$i/$j" >  mirrors-epages-arch-$i-$j
      echo "http://ep-server/RPMS/kernel/$i/$j" >>  mirrors-epages-arch-$i-$j
    fi
  done
done

# create epages-base.repo
repo=epages-base.repo

if [[ ! -f $repo ]] ; then
  echo '### Name: ePages RPM Repository' > $repo
  echo '### URL: http://epages.com/' >> $repo
  echo -e "[epages-base]\nname = epages.com - base" >> $repo
  echo -e "baseurl = $EPAGES_RPMS_URL/$INSTALL_VER/base/$KERNEL" >> $repo
  echo "mirrorlist = file:///etc/yum.repos.d/mirrors-epages-base-$KERNEL" >> $repo
  echo -e "enabled = 1\ntype = rpm-md\ngpgcheck = 1" >> $repo
  echo -e "gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-epages" >> $repo
fi
for j in 2.6.16s 2.6.18r 2.6.27s 2.6.32r 2.6.32s ; do
  if [[ ! -f mirrors-epages-base-$j ]] ; then
    echo "http://www.epages.com/RPMS/$INSTALL_VER/base/$j" >  mirrors-epages-base-$j
    echo "http://ep-server/RPMS/$INSTALL_VER/base/$j" >>  mirrors-epages-base-$j
  fi
done

# create epages-setup.repo
repo=epages-setup.repo

if [[ ! -f $repo || $INTERNAL_INSTALL_VER -lt 6014000 ]] ; then
  echo '### Name: ePages RPM Repository' > $repo
  echo '### URL: http://epages.com/' >> $repo
  echo -e "[epages-setup]\nname = epages.com - setup" >> $repo
  echo -e "baseurl = $EPAGES_RPMS_URL/$INSTALL_VER/\$basearch" >> $repo
  echo "mirrorlist = file:///etc/yum.repos.d/mirrors-epages-setup" >> $repo
  echo -e "enabled = 0\ntype = rpm-md\ngpgcheck = 1" >> $repo
  echo -e "gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-epages" >> $repo
fi
if [[ ! -f mirrors-epages-setup || $INTERNAL_INSTALL_VER -lt 6014000 ]] ; then
  echo "http://www.epages.com/RPMS/$INSTALL_VER/\$ARCH" >  mirrors-epages-setup
  echo "http://ep-server/RPMS/$INSTALL_VER/\$ARCH" >>  mirrors-epages-setup
fi

# create epages-update.repo
repo=epages-update.repo

echo '### Name: ePages RPM Repository' > $repo
echo '### URL: http://epages.com/' >> $repo
echo -e "[epages-update]\nname = epages.com - update" >> $repo
echo -e "baseurl = $EPAGES_RPMS_URL/$UPDATE_VER-update" >> $repo
echo "mirrorlist = file:///etc/yum.repos.d/mirrors-epages-update" >> $repo
echo -e "enabled = 0\ntype = rpm-md\ngpgcheck = 1" >> $repo
echo -e "gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-epages" >> $repo

echo "http://www.epages.com/RPMS/$UPDATE_VER-update" >  mirrors-epages-update
echo "http://ep-server/RPMS/$UPDATE_VER-update" >>  mirrors-epages-update

# create epages-empty.repo
repo=epages-empty.repo

sed "s/update/empty/" epages-update.repo > $repo
sed "s/update/empty/" mirrors-epages-update > mirrors-epages-empty

# create epages-archive.repo
repo=epages-archive.repo

echo '### Name: ePages RPM Repository' > $repo
echo '### URL: http://epages.com/' >> $repo
for MINOR in {10..20} ; do
  for LEVEL in {0..4} ; do
    echo -e "\n[epages-6.$MINOR.$LEVEL]\nname = epages.com - 6.$MINOR.$LEVEL"
    echo -e "baseurl = $EPAGES_RPMS_URL/6.$MINOR.$LEVEL/\$basearch"
    echo -e "enabled = 0\ntype = rpm-md\ngpgcheck = 1"
    echo -e "gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-epages"
  done
done >> $repo

# create epages-update-archive.repo
repo=epages-update-archive.repo

echo '### Name: ePages RPM Repository' > $repo
echo '### URL: http://epages.com/' >> $repo
for MINOR in {10..20} ; do
  for LEVEL in {0..4} ; do
    echo -e "\n[epages-update-6.$MINOR.$LEVEL]\nname = epages.com - 6.$MINOR.$LEVEL-update"
    echo -e "baseurl = $EPAGES_RPMS_URL/6.$MINOR.$LEVEL/$UPDATE_VER-update"
    echo -e "enabled = 0\ntype = rpm-md\ngpgcheck = 1"
    echo -e "gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-epages"
  done
done >> $repo

# -----------------------------------------------------------------------------
# edit repositories
# -----------------------------------------------------------------------------

# enable/disable repositories

sed -ri 's,^(enabled).*,\1 = 0,' epages-empty.repo || exit 1
sed -ri 's,^(enabled).*,\1 = 0,' epages-update.repo || exit 1

if [[ -n $EPAGES_INSTALL_VERSION || -n $COMMON_RPM_INSTALLED  ]] ; then
  sed -ri 's,^(enabled).*,\1 = 0,' epages-setup.repo || exit 1
else
  sed -ri 's,^(enabled).*,\1 = 1,' epages-setup.repo || exit 1
fi

if [[ -n $EPAGES_INSTALL_VERSION && -z $COMMON_RPM_INSTALLED  ]] ; then
  PATH=/var/epages:$PATH:/var/tmp:"$EPAGES"/bin
  ini-config.sh -f epages-archive.repo -s epages-$TARGET_VER -k enabled -v 1 -i || exit 1
fi

# installed epages version < 6.9 -> no repo exists
# installed epages version < 6.13 and target version >= 6.14 -> . /var/epages/functions.sh

if [[ -n $SOURCE_VER ]] ; then
  INTERNAL_SOURCE_VER=$(printf "%d%03d%03d" ${SOURCE_VER//./ })
  if [[ $INTERNAL_SOURCE_VER -lt 6009000 ]] ; then
    for j in epages*.repo ; do
      sed -ri 's,^(enabled).*,\1 = 0,' $j || exit 1
    done
  fi

  if [[ -f /etc/default/$EPAGES_PROJECT ]] ; then
    if [[ $INTERNAL_TARGET_VER -ge 6014000 && $INTERNAL_SOURCE_VER -lt 6013000 ]] ; then
      grep -q '\. /var/epages/functions.sh' /etc/default/$EPAGES_PROJECT \
      || echo '. /var/epages/functions.sh' >> /etc/default/$EPAGES_PROJECT
    fi
  fi
fi

# set current version in epages-update.repo and epages-empty.repo

if [[ -n $SOURCE_VER ]] ; then
  for repo in update empty ; do
    for file in epages-$repo.repo mirrors-epages-$repo ; do
      [[ -f $file ]] || continue
      sed -ri "s,/[6-9]\.[0-9]+-$repo,/$UPDATE_VER-$repo," $file
    done
  done
fi

# set target version in epages-setup.repo

if [[ -n $EPAGES_INSTALL_VERSION ]] ; then
  for file in epages-setup.repo mirrors-epages-setup ; do
    [[ -f $file ]] || continue
    sed -ri "s,/[6-9]\.[0-9]+\.[0-9]+/,/$TARGET_VER/," $file
  done
fi

# set own repository

if [[ -n $OWN_REPO ]] ; then
  sed -ri "s,^(\s*baseurl\b).*/([6-9]\.[0-9]+\.[0-9]+/.*),\1 = $EPAGES_RPMS_URL/\2," epages-setup.repo
  sed -ri "s,^(\s*baseurl\b).*/([6-9]\.[0-9]+-update.*),\1 = $EPAGES_RPMS_URL/\2," epages-update.repo
  sed -ri "s,^(\s*baseurl\b).*/([6-9]\.[0-9]+-empty.*),\1 = $EPAGES_RPMS_URL/\2," epages-empty.repo
  sed -ri "s,^(\s*baseurl\b).*/([6-9]\.[0-9]+\.[0-9]+/base/.*),\1 = $EPAGES_RPMS_URL/\2," epages-base.repo

  echo "$EPAGES_RPMS_URL/$INSTALL_VER/\$ARCH" > mirrors-epages-setup
  echo "$EPAGES_RPMS_URL/$UPDATE_VER-update" > mirrors-epages-update
  echo "$EPAGES_RPMS_URL/$UPDATE_VER-empty" > mirrors-epages-empty
  echo "$EPAGES_RPMS_URL/$INSTALL_VER/base/$KERNEL" >  mirrors-epages-base-$KERNEL

  sed -ri "s,^(\s*baseurl\b).*/(kernel/.*),\1 = $EPAGES_RPMS_URL/\2," epages-arch.repo
  echo "$EPAGES_RPMS_URL/kernel/$ARCH" > mirrors-epages-arch-$ARCH
  echo "$EPAGES_RPMS_URL/kernel/$ARCH/$KERNEL" > mirrors-epages-arch-$ARCH-$KERNEL
  if [[ $ARCH != i386 ]] ; then
    echo "$EPAGES_RPMS_URL/kernel/i386" > mirrors-epages-arch-i386
  fi
fi

chmod 644 *epages*

# -----------------------------------------------------------------------------
# convert yum repositories to zypper repositories
# -----------------------------------------------------------------------------

add_zypper_repos epages*.repo

if [[ $INSTALLER = yum ]] ; then
  pgrep yum | grep -q . || yum clean metadata
else
  pgrep zypper | grep -q . || zypper clean -m 2>/dev/null
fi

for RPM in $OTHER_RPMS ; do
  rpm --quiet -q $RPM && continue
  echo "$INSTALLER $YES install $ZES $RPM"
  $INSTALLER $YES install $ZES $RPM || exit 1
done

grep -q 'error:' /tmp/${BN%.sh}.log 2>/dev/null && exit 1 || exit 0

# -----------------------------------------------------------------------------
# exit
# -----------------------------------------------------------------------------
